Package: permute
Title: Functions for Generating Restricted Permutations of Data
Version: 0.9-10
Authors@R: c(person(given = "Gavin L.", family = "Simpson",
                    email = "ucfagls@gmail.com",
                    role = c("aut", "cph", "cre"),
		                comment = c(ORCID = "0000-0002-9084-8413")),
             person(given = "R Core Team", role = "cph"),
             person(given = "Douglas M.", family = "Bates", role = "ctb"),
             person(given = "Jari", family = "Oksanen", role = "ctb"))
Depends: R (>= 3.6.0)
Imports: stats
Suggests: vegan (>= 2.0-0), testthat (>= 0.5), parallel, knitr,
        rmarkdown, bookdown, sessioninfo
Description: A set of restricted permutation designs for freely exchangeable, line transects (time series), and spatial grid designs plus permutation of blocks (groups of samples) is provided. 'permute' also allows split-plot designs, in which the whole-plots or split-plots or both can be freely-exchangeable or one of the restricted designs. The 'permute' package is modelled after the permutation schemes of 'Canoco 3.1' (and later) by Cajo ter Braak.
License: GPL-2
ByteCompile: true
URL: https://github.com/gavinsimpson/permute
BugReports: https://github.com/gavinsimpson/permute/issues
Copyright: see file COPYRIGHTS
VignetteBuilder: knitr
Language: en-GB
NeedsCompilation: no
Packaged: 2026-02-06 11:02:27 UTC; gavin
Author: Gavin L. Simpson [aut, cph, cre] (ORCID:
    <https://orcid.org/0000-0002-9084-8413>),
  R Core Team [cph],
  Douglas M. Bates [ctb],
  Jari Oksanen [ctb]
Maintainer: Gavin L. Simpson <ucfagls@gmail.com>
Repository: CRAN
Date/Publication: 2026-02-06 11:50:02 UTC

### Visible functions:
export("allPerms",
       "as.allPerms",
       "Blocks",
       "check",
       "getBlocks",
       "getWithin",
       "getStrata",
       "getType",
       "getMirror",
       "getConstant",
       "getPlots",
       "getRow",
       "getCol",
       "getDim",
       "getNperm",
       "getMaxperm",
       "getMinperm",
       "getComplete",
       "getMake",
       "getObserved",
       "getAllperms",
       "getControl",
       "getHow",
       "how",
       "numPerms",
       "permute",
       "Plots",
       "setBlocks<-",
       "setWithin<-",
       "setStrata<-",
       "setType<-",
       "setMirror<-",
       "setConstant<-",
       "setPlots<-",
       "setRow<-",
       "setCol<-",
       "setDim<-",
       "setNperm<-",
       "setMaxperm<-",
       "setMinperm<-",
       "setComplete<-",
       "setMake<-",
       "setObserved<-",
       "setAllperms<-",
       "shuffle",
       "shuffleFree",
       "shuffleGrid",
       "shuffleSeries",
       "shuffleSet",
       "shuffleStrata",
       "Within"
       )

### Imports: nobs() only exists in R 2.13.0 for import. We define the
### same nobs() generic in permute for export in older R.
if (getRversion() >= "2.13.0") {
    importFrom("stats", "nobs")
} else {
    export(nobs)
}

### After R 3.2.x, R-Devel required all non-Base functions to be
### imported
importFrom("stats", "getCall", "runif", "update")

### S3 Methods
S3method("as.matrix", "allPerms")
S3method("as.matrix", "permutationMatrix")
S3method("update", "how")
S3method("update", "Plots")
## print methods
S3method("print", "allPerms")
S3method("print", "check")
S3method("print", "how")
S3method("print", "permutationMatrix")
S3method("print", "summary.allPerms")
S3method("print", "summary.check")
## summary methods
S3method("summary", "allPerms")
S3method("summary", "check")
## nobs() methods
S3method("nobs", "numeric")
S3method("nobs", "integer")
S3method("nobs", "matrix")
S3method("nobs", "data.frame")
S3method("nobs", "factor")
S3method("nobs", "character")
## getFoo methods
S3method("getBlocks", "default")
S3method("getBlocks", "how")
S3method("getPlots", "default")
S3method("getPlots", "how")
S3method("getWithin", "default")
S3method("getWithin", "how")
S3method("getStrata", "default")
S3method("getStrata", "how")
S3method("getStrata", "Plots")
S3method("getType", "default")
S3method("getType", "how")
S3method("getType", "Plots")
S3method("getType", "Within")
S3method("getMirror", "default")
S3method("getMirror", "how")
S3method("getMirror", "Plots")
S3method("getMirror", "Within")
S3method("getConstant", "default")
S3method("getConstant", "how")
S3method("getConstant", "Within")
S3method("getNperm", "default")
S3method("getNperm", "how")
S3method("getMaxperm", "default")
S3method("getMaxperm", "how")
S3method("getMinperm", "default")
S3method("getMinperm", "how")
S3method("getComplete", "default")
S3method("getComplete", "how")
S3method("getRow", "default")
S3method("getRow", "how")
S3method("getRow", "Plots")
S3method("getRow", "Within")
S3method("getCol", "default")
S3method("getCol", "how")
S3method("getCol", "Plots")
S3method("getCol", "Within")
S3method("getDim", "default")
S3method("getDim", "how")
S3method("getDim", "Plots")
S3method("getDim", "Within")
S3method("getMake", "default")
S3method("getMake", "how")
S3method("getObserved", "default")
S3method("getObserved", "how")
S3method("getAllperms", "default")
S3method("getAllperms", "how")
S3method("getControl", "default")
S3method("getControl", "allPerms")

## setFoo methods
S3method("setBlocks<-", "default")
S3method("setBlocks<-", "how")
S3method("setPlots<-", "default")
S3method("setPlots<-", "how")
S3method("setWithin<-", "default")
S3method("setWithin<-", "how")
S3method("setStrata<-", "default")
S3method("setStrata<-", "how")
S3method("setStrata<-", "Plots")
S3method("setType<-", "default")
S3method("setType<-", "how")
S3method("setType<-", "Within")
S3method("setType<-", "Plots")
S3method("setMirror<-", "default")
S3method("setMirror<-", "how")
S3method("setMirror<-", "Within")
S3method("setMirror<-", "Plots")
S3method("setConstant<-", "default")
S3method("setConstant<-", "how")
S3method("setConstant<-", "Within")
S3method("setConstant<-", "Plots")
S3method("setNperm<-", "default")
S3method("setNperm<-", "how")
S3method("setMaxperm<-", "default")
S3method("setMaxperm<-", "how")
S3method("setMinperm<-", "default")
S3method("setMinperm<-", "how")
S3method("setComplete<-", "default")
S3method("setComplete<-", "how")
S3method("setRow<-", "default")
S3method("setRow<-", "how")
S3method("setRow<-", "Within")
S3method("setRow<-", "Plots")
S3method("setCol<-", "default")
S3method("setCol<-", "how")
S3method("setCol<-", "Within")
S3method("setCol<-", "Plots")
S3method("setDim<-", "default")
S3method("setDim<-", "how")
S3method("setDim<-", "Within")
S3method("setDim<-", "Plots")
S3method("setMake<-", "default")
S3method("setMake<-", "how")
S3method("setObserved<-", "default")
S3method("setObserved<-", "how")
S3method("setAllperms<-", "default")
S3method("setAllperms<-", "how")

## Restricted permutations with R

[![CRAN version](https://www.r-pkg.org/badges/version/permute)](https://cran.r-project.org/package=permute)
[![](https://cranlogs.r-pkg.org/badges/grand-total/permute)](https://cran.r-project.org/package=permute)
[![R-CMD-check](https://github.com/gavinsimpson/permute/workflows/R-CMD-check/badge.svg)](https://github.com/gavinsimpson/cocorresp/actions)
[![codecov](https://codecov.io/gh/gavinsimpson/permute/graph/badge.svg?token=2FYEfBBSJ7)](https://app.codecov.io/gh/gavinsimpson/permute)

## What is permute?

**permute** generates permutations from a range of restricted 
permutation designs.

Permute provides an R implementation of the permutation schemes 
developed by Cajo ter Braak and made available in the Canoco software, 
version 3.1 (ter Braak, 1990). These permutation schemes draw upon 
ideas from an earlier paper by Besag & Clifford (1989).

Several types of permutation are available in **permute**:

 * Free permutation of objects
 * Time series or line transect designs, where the temporal or spatial ordering is preserved.
 * Spatial grid designs, where the spatial ordering is preserved in both coordinate directions
 * Permutation of plots or groups of samples.
 * Blocking factors which restrict permutations to within blocks. The preceding designs can be nested within blocks, allowing analysis of hierarchical designs (e.g. split plot designs)

### References

Besag, J. and Clifford, P. (1989) Generalized Monte Carlo significance tests. *Biometrika* **76**; 633&ndash;642.

ter Braak, C. J. F. (1990). *Update notes: CANOCO version 3.1*. Wageningen: Agricultural Mathematics Group. (UR).

## new version of shuffleSet() that allows for blocking
`shuffleSet` <- function(n, nset, control = how(), check = TRUE,
                         quietly = FALSE) {
    ## Store the .Random.seed, if it exists, so we can attach this as
    ## an attribute to the permutation matrix returned in out
    SEED <- NULL
    if (exists(".Random.seed", envir = globalenv())) {
        SEED <- .Random.seed
    }

    ## handle missing nset - take from control if can
    if(missing(nset)) {
        np <- getNperm(control)
        if(is.null(np)) ## something wrong, default back to 1
            nset <- 1
        else
            nset <- np
    } else {
        setNperm(control) <- nset ## this fixes the control$call too!
    }

    ## handle a vector, matrix, or data frame input; derive n from it
    if (((is.numeric(n) || is.integer(n) || is.factor(n) || is.character(n)) &&
         length(n) > 1L) ||
        is.matrix(n) ||
        is.data.frame(n)) {
        n <- nobs(n)
    }
    sn <- seq_len(n) ## sequence of samples in order of input

    ## if checking permutation design, may end up with more perms
    ## than requested in nset, depending upon what user specified
    ## in `control`. The `check` argument can turn this step off
    ## so you always get `nset` permutations and, yes, you can shoot
    ## yourself in the foot with this, hence the default is to check!
    if (isTRUE(check)) {
        ## store original complete
        all.orig <- getComplete(control)
        ## need to check number of permutations won't blow up
        pcheck <- check(sn, control = control, quietly = quietly)
        ## control possibly now updated
        control <- getControl(pcheck)
        ## if complete != TRUE originally, it must have been triggered by
        ## small permutation size. If so, don't change nset. Only change if
        ## user asked for complete = TRUE in the first place!
        if (isTRUE(all.orig)) {
            nset <- getNperm(control)       # update nset as it might have changed
        }
    }

    if(is.null(AP <- getAllperms(control))) {
        ## get blocking, if any
        Block <- getStrata(control, which = "blocks")
        if(is.null(Block))
            Block <- factor(rep(1, n))

        ## split sn on basis of Block
        spln <- split(sn, Block)
        nb <- length(spln) ## number of blocks

        ## result list
        out <- vector(mode = "list", length = nb)

        ## loop over spln and shuffle in each split
        for(i in seq_len(nb)) {
            out[[i]] <- doShuffleSet(spln[[i]], nset = nset, control)
        }
        ## undo the original splitting. Can't use unsplit() here as the
        ## elements of out are matrices
        out <- do.call(cbind, out)
        out[, unlist(spln)] <- out ## reorders according to spln
    } else {
        ## if we have all.perms now then we must have generated it
        ## during checking or user passed it with control
        ## Use that instead of a ranodm set
        out <- AP
    }

    ## Because all.perms might have been generated, we have the
    ## possibility that nrow(out) != nset. In that case, also no random
    ## numbers have been generated. Hence we can sample nset rows from
    ## out and return that. This has the nice side-effect of not
    ## generating any non-unique permutations. Suggested by Jari.
    if ((nr <- nrow(out)) > nset) {
        out <- out[sample.int(nr, nset), , drop = FALSE]
    }

    ## Attach random seed stored earlier to permutation matrix
    attr(out, "seed") <- SEED
    attr(out, "control") <- control
    attr(out, "observed") <- NULL ## nullify this as allPerms may have added it?

    ## class the matrix so we can have a print method etc, but inherit from
    ## the matrix S3 class
    class(out) <- c("permutationMatrix", "matrix")

    ## return
    out
}

`doShuffleSet` <- function(ind, nset = 1, control) {
    ## collect strata at Plot level
    Pstrata <- getStrata(control, which = "plots", drop = TRUE)
    plotCTRL <- getPlots(control)
    typeP <- getType(control, which = "plots")

    ## collect the within control object
    withinCTRL <- getWithin(control)
    typeW <- getType(control, which = "within")

    n <- length(ind)
    sn <- seq_len(n)

    ## result object
    Set <- matrix(nrow = nset, ncol = n)

    ## if no strata at Plot level permute all samples using stated scheme
    if(is.null(Pstrata)) {
        ## If no strata at plot then permute all samples using stated scheme
        Args <- switch(typeW,
                       "free" = list(x = n, size = n),
                       "series" = list(x = seq_len(n),
                           mirror = withinCTRL$mirror),
                       "grid" = list(nrow = withinCTRL$nrow,
                           ncol = withinCTRL$ncol,
                           mirror = withinCTRL$mirror))
        FUN <- switch(typeW,
                      "free" = shuffleFree,
                      "series" = shuffleSeries,
                      "grid" = shuffleGrid)
        if(withinCTRL$type == "none") {
            Set[] <- rep(sn, each = nset)
        } else {
            for(i in seq_len(nset)) {
                Set[i,] <- do.call(FUN, Args)
            }
        }
    } else {
        ## If strata at Plot level present, either permute samples, Plots or both

        ## permute strata at Plot level?
        if(isTRUE(all.equal(typeP, "none"))) {
            Set[] <- rep(sn, each = nset)
        } else {
            for(i in seq_len(nset)) {
                Set[i,] <- do.call(shuffleStrata,
                                   list(strata = Pstrata[ind],
                                        type = typeP,
                                        mirror = plotCTRL$mirror,
                                        flip = NULL, ## runif(1L) < 0.5 ??
                                        nrow = plotCTRL$nrow,
                                        ncol = plotCTRL$ncol))
            }
        }

        tmp <- Set

        ## permute the samples within Plot strata
        if(!isTRUE(all.equal(typeW, "none"))) {
            for(i in seq_len(nset)) {
                tab <- table(Pstrata[ind][Set[i,]])
                ## the levels of the Plot strata
                levs <- names(tab)

                ## same permutation within each level of the Plot strata?
                if(withinCTRL$constant) {
                    if(isTRUE(all.equal(typeW, "free"))) {
                        n <- unique(tab)[1L]
                        same.rand <- shuffleFree(n, n)
                    } else if(isTRUE(all.equal(typeW, "series"))) {
                        start <- shuffleFree(n / length(levs), 1L)
                        flip <- runif(1L) < 0.5 ## FIXME this should be moved out of the loop
                    } else if(isTRUE(all.equal(typeW, "grid"))) {
                        start.row <- shuffleFree(withinCTRL$nrow, 1L)
                        start.col <- shuffleFree(withinCTRL$ncol, 1L)
                        flip <- runif(2L) < 0.5 ## FIXME this should be moved out of the loop
                    }
                } else {
                    start <- start.row <- start.col <- flip <- NULL
                }

                ## for each level of strata, permute
                for(lv in levs) {
                    ## must re-order strata here on basis of Ser as they
                    ## may have been permuted above
                    MATCH <- Pstrata[ind][Set[i,]] == lv
                    gr <- Set[i,][MATCH]
                    if((n.gr <- length(gr)) > 1) {
                        if(withinCTRL$constant && isTRUE(all.equal(typeW, "free"))) {
                            tmp[i,][which(MATCH)] <- gr[same.rand]
                        } else {
                            Args <-
                                switch(typeW,
                                       "free" = list(x = n.gr, size = n.gr),
                                       "series" = list(x = seq_len(n.gr),
                                           mirror = withinCTRL$mirror,
                                           start = start,
                                           flip = flip),
                                       "grid" = list(nrow = withinCTRL$nrow,
                                           ncol = withinCTRL$ncol,
                                           mirror = withinCTRL$mirror,
                                           start.row = start.row,
                                           start.col = start.col,
                                           flip = flip))
                            FUN <-
                                switch(typeW,
                                       "free" = shuffleFree,
                                       "series" = shuffleSeries,
                                       "grid" = shuffleGrid)
                            tmp[i,][which(MATCH)] <- gr[do.call(FUN, Args)]
                        }
                    }
                }
            }
            Set <- tmp
        }
    }
    out <- Set ## have to copy or next line fails
    out[] <- ind[Set]
    out
}

library("testthat")
library("permute")

context("Testing as.foo() methods")

test_that("as.matrix allPerms method", {
    ap <- allPerms(1:3)
    m <- as.matrix(ap)
    expect_is(m, "matrix")
    expect_false(inherits(m, "allPerms"))
})

test_that("as.matrix permutationMatrix method", {
    perms <- shuffleSet(10, nset = 10)
    m <- as.matrix(perms)
    expect_is(m, "matrix")
    expect_false(inherits(m, "permutationMatrix"))
})

test_that("as.allPerms fixes #16", {
    res1 <- check(4, control = how())
    ctrl <- getControl(res1)
    res <- check(4, ctrl)
    expect_is(res, "check")
})

library("testthat")
library("permute")

context("Testing shuffleSet()")

## test that shuffleSet interleves the separate block-level
## permutations correctly back into the original ordering
## This always generates odd, even, odd, ..., numbered vector
## of observations, hence when we take the modulus 2 we get
## a vector of 1,0,1,0,1,...
test_that("shuffleSet interleves block-level perms correctly", {
    gr <- factor(rep(1:2, length=20))
    ctrl <- how(nperm = 5, blocks = gr)
    p <- shuffleSet(20, control = ctrl) %% 2
    y <- rep(c(1L, 0L), length.out = ncol(p))
    nc <- ncol(p)
    for (i in seq_len(nrow(p))) {
        expect_that(p[i, ], equals(y))
    }
})

## test that nset permutations are always returned if
## make = FALSE in how()
test_that( "shuffleSet returns exactly nset permutations when make == FALSE", {
    ## simple random permutation
    h <- how(make = FALSE)
    ss <- shuffleSet(n = 4, nset = 10, control = h)
    expect_that(nrow(ss), equals(10))

    ## time series
    h <- how(within = Within(type = "series"), make = FALSE)
    ss <- shuffleSet(n = 20, nset = 15, control = h)
    expect_that(nrow(ss), equals(15))

})

## test that shuffleSet always returns a matrix, even for nset == 1
test_that("shuffleSet returns a matrix even for nset == 1", {
    h <- how()
    ss <- shuffleSet(25, nset = 1, control = h)
    expect_that(ss, is_a("matrix"))
})

test_that("shuffle can permute both plots and within in presence of blocks", {
    ## Example from @LindsayVass on github #9
    control <- how(within = Within(type = "free"),
                   plots = Plots(strata = rep(gl(2,7),2), type = "free"),
                   blocks = gl(2, 14))
    permSet <- shuffleSet(28, 100, control = control)
    expect_that(nrow(permSet), is_identical_to(100L))
    expect_that(ncol(permSet), is_identical_to(28L))
    expect_that(permSet, is_a("permutationMatrix"))
    expect_that(permSet, is_a("matrix"))
})

test_that("print method for permutationMatrix works", {
    h <- how()
    perms <- shuffleSet(10, nset = 10, control = h)
    expect_output(print(perms), regexp = "No. of Permutations:")

    h <- how(blocks = gl(5,10))
    perms <- shuffleSet(50, nset = 20, control = h)
    expect_output(print(perms), regexp = "Restricted by Blocks:")
    expect_output(print(perms), regexp = "blocks;")

    h <- how(plots = Plots(strata = gl(5,10)))
    perms <- shuffleSet(50, nset = 20, control = h)
    expect_output(print(perms), regexp = "Restricted by Plots:")
    expect_output(print(perms), regexp = "plots;")

    h <- how(plots = Plots(strata = gl(10,10)),
             blocks = gl(2, 50))
    perms <- shuffleSet(100, nset = 20, control = h)
    expect_output(print(perms), regexp = "Restricted by Plots:")
    expect_output(print(perms), regexp = "plots & blocks;")

    h <- how(within = Within(type = "series", mirror = TRUE))
    perms <- shuffleSet(10, nset = 20, control = h)
    expect_output(print(perms), regexp = "; mirrored")

    h <- how(within = Within(type = "series", constant = TRUE),
             plots = Plots(strata = gl(2, 5), type = "series", mirror = TRUE))
    perms <- shuffleSet(10, nset = 20, control = h)
    expect_output(print(perms), regexp = "; same permutation")
})

test_that("constant within plots works", {
    ## These need check = FALSE to stop allPerms doing the generation
    ## which doesn't actually call doShuffleSet. Check using both FALSE
    ## and the default, default first
    fac <- gl(4, 5)
    nset <- 10L
    ## series permutations
    ctrl <- how(within = Within(type = "series", constant = TRUE),
                plots = Plots(strata = fac, type = "none"))
    perm <- shuffleSet(length(fac), nset, control = ctrl)
    expect_identical(ncol(perm), length(fac))
    expect_identical(ncol(perm), 20L)
    expect_identical(nrow(perm), 4L) ## only 4 permutations!
    expect_is(perm, "matrix")
    expect_is(perm, "permutationMatrix")
    expect_output(print(perm), regexp = "; same permutation")
    expect_output(print(perm), regexp = "Nested in: plots; Sequence;")

    ## free/randomisation
    ctrl <- how(within = Within(type = "free", constant = TRUE),
                plots = Plots(strata = fac, type = "none"))
    perm <- shuffleSet(length(fac), nset, control = ctrl)
    expect_identical(ncol(perm), length(fac))
    expect_identical(ncol(perm), 20L)
    expect_identical(nrow(perm), nset)
    expect_identical(nrow(perm), 10L)
    expect_is(perm, "matrix")
    expect_is(perm, "permutationMatrix")
    expect_output(print(perm), regexp = "; same permutation")
    expect_output(print(perm), regexp = "Nested in: plots; Randomised;")

    ## spatial grid 3x3
    fac <- gl(4, 9)
    ctrl <- how(within = Within(type = "grid", nrow = 3, ncol = 3, constant = TRUE),
                plots = Plots(strata = fac, type = "none"))
    perm <- shuffleSet(length(fac), nset, control = ctrl)
    expect_identical(ncol(perm), length(fac))
    expect_identical(ncol(perm), 36L)
    expect_identical(nrow(perm), 8L)
    expect_is(perm, "matrix")
    expect_is(perm, "permutationMatrix")
    expect_output(print(perm), regexp = "; same\npermutation")
    expect_output(print(perm), regexp = "; Spatial grid: 3r, 3c")

    ## now with check = FALSE --- this is going to generate duplicate permutations
    fac <- gl(4, 5)
    nset <- 10L
    ## series permutations
    ctrl <- how(within = Within(type = "series", constant = TRUE),
                plots = Plots(strata = fac, type = "none"))
    perm <- shuffleSet(length(fac), nset, control = ctrl, check = FALSE)
    expect_identical(ncol(perm), length(fac))
    expect_identical(ncol(perm), 20L)
    expect_identical(nrow(perm), nset)
    expect_identical(nrow(perm), 10L)
    expect_is(perm, "matrix")
    expect_is(perm, "permutationMatrix")
    expect_output(print(perm), regexp = "; same permutation")
    expect_output(print(perm), regexp = "Nested in: plots; Sequence;")

    ## free/randomisation
    ctrl <- how(within = Within(type = "free", constant = TRUE),
                plots = Plots(strata = fac, type = "none"))
    perm <- shuffleSet(length(fac), nset, control = ctrl, check = FALSE)
    expect_identical(ncol(perm), length(fac))
    expect_identical(ncol(perm), 20L)
    expect_identical(nrow(perm), nset)
    expect_identical(nrow(perm), 10L)
    expect_is(perm, "matrix")
    expect_is(perm, "permutationMatrix")
    expect_output(print(perm), regexp = "; same permutation")
    expect_output(print(perm), regexp = "Nested in: plots; Randomised;")

    ## spatial grid 3x3
    fac <- gl(4, 9)
    ctrl <- how(within = Within(type = "grid", nrow = 3, ncol = 3, constant = TRUE),
                plots = Plots(strata = fac, type = "none"))
    perm <- shuffleSet(length(fac), nset, control = ctrl, check = FALSE)
    expect_identical(ncol(perm), length(fac))
    expect_identical(ncol(perm), 36L)
    expect_identical(nrow(perm), 10L)
    expect_identical(nrow(perm), nset)
    expect_is(perm, "matrix")
    expect_is(perm, "permutationMatrix")
    expect_output(print(perm), regexp = "; same\npermutation")
    expect_output(print(perm), regexp = "; Spatial grid: 3r, 3c")
})

test_that("shuffelSet works with objects passed to n", {
    obj <- 1:4
    p <- shuffleSet(obj, 10L)
    expect_is(p, "permutationMatrix")
    expect_identical(nrow(p), 10L)
    obj <- as.integer(1:4)
    p <- shuffleSet(obj, 10L)
    expect_is(p, "permutationMatrix")
    expect_identical(nrow(p), 10L)
    obj <- as.factor(1:4)
    p <- shuffleSet(obj, 10L)
    expect_is(p, "permutationMatrix")
    expect_identical(nrow(p), 10L)
    obj <- letters[1:4]
    p <- shuffleSet(obj, 10L)
    expect_is(p, "permutationMatrix")
    expect_identical(nrow(p), 10L)
    obj <- matrix(1:16, ncol = 4, nrow = 4)
    p <- shuffleSet(obj, 10L)
    expect_is(p, "permutationMatrix")
    expect_identical(nrow(p), 10L)
    obj <- data.frame(A = 1:4, B = letters[1:4])
    p <- shuffleSet(obj, 10L)
    expect_is(p, "permutationMatrix")
    expect_identical(nrow(p), 10L)
})

test_that("Issue 19: shuffleSet with nset=1 never regresses", {
    TreatmentLevels <- 3
    Animals <- 4
    TimeSteps <- 5
    AnimalID <- rep(letters[seq_len(Animals)], each = TreatmentLevels * TimeSteps)
    Time <- rep(seq_len(TimeSteps), Animals = TreatmentLevels)
    ## Treatments were given in different order per animal
    Day <- rep(c(1,2,3,2,3,1,3,2,1,2,3,1), each = TimeSteps)
    Treatment <- rep(rep(LETTERS[seq_len(TreatmentLevels)], each = TimeSteps), Animals)
    dataset <- as.data.frame(cbind(AnimalID, Treatment, Day, Time))

    ctrl <- with(dataset,
                 how(blocks = AnimalID, plots = Plots(strata = Day, type = "free"),
                     within = Within(type = "none"), nperm = 999))

    ## This should return a matrix
    p <- shuffleSet(60, nset = 1, control = ctrl)
    expect_is(p, "matrix")
    expect_identical(nrow(p), 1L)
})

test_that("Issue 23: if complete = TRUE update nperm", {
    ctrl <- how(complete = TRUE, minperm = 100)
    vec <- 1:6
    np <- numPerms(vec, control = ctrl)
    ap <- allPerms(vec, control = ctrl)
    p <- shuffleSet(length(vec), control = ctrl)
    ##expect_identical(NROW(p), np - 1L)
    expect_identical(NROW(p), NROW(ap))
})

#library("testthat")
#library("permute")

context("Testing shuffleSet()")

test_that("numPerms works with complex, but small, design", {
    h <- how(within = Within(type = "series", constant = TRUE),
             plots = Plots(strata = gl(2, 5), type = "series", mirror = TRUE))
    np <- numPerms(10, control = h)
    expect_type(np, "double")
    expect_equal(np, 10L)
})

test_that("numPerms doesn't suffer floating point probs as issue 41", {
    y1 <- rnorm(150)
    f <- as.factor(c(
        rep(0, 30),
        rep(1, 30),
        rep(2, 30),
        rep(3, 30),
        rep(4, 30))
    )
    h <- how(
        within = Within(type = "none"),
        plots = Plots(strata = f, type = "free")
    )

    np <- numPerms(y1, control = h)
    expect_type(np, "double")
    expect_equal(np, 120L)

    np <- permute::check(y1, h, quietly = TRUE)
    expect_type(np$n, "double")
    expect_equal(np$n, 120L)
})

`Within` <- function(type = c("free","series","grid","none"),
                     constant = FALSE, mirror = FALSE,
                     ncol = NULL, nrow = NULL)
{
    type <- match.arg(type)

    ## process the call to make it standalone
    .call <- match.call()
    if (length(.call) > 1L) {
        .ll <- as.list(.call[-1])
        for (i in seq_along(.ll))
            .ll[[i]] <- eval(.ll[[i]], parent.frame())
        .ll <- c(as.list(.call[[1]]), .ll)
        names(.ll) <- names(.call)
        .call <- as.call(.ll)
    }

    out <- list(type = type, constant = constant, mirror = mirror,
                ncol = ncol, nrow = nrow, call = .call)
    class(out) <- "Within"
    out
}

library("testthat")
library("permute")

context("Testing check()")

## test that check will return all perms including the observed
test_that("check returns observed ordering in set of all permutations *if* asked to", {
    ## simple permutation
    h <- how(observed = TRUE)
    pp <- check(4, control = h)
    expect_that(nrow(pp$control$all.perms), equals(factorial(4)))

    ## time series
    h <- how(within = Within(type = "series"), observed = TRUE)
    n <- 10
    pp <- check(n, control = h)
    expect_that(nrow(pp$control$all.perms), equals(n))

    ## time series specified as a vector
    h <- how(within = Within(type = "series"), observed = TRUE)
    n <- 10
    vec <- seq_len(n)
    pp <- check(vec, control = h)
    expect_that(nrow(pp$control$all.perms), equals(n))
})

## test that check throws an error when within and plot permutation
## type is "none"
test_that("check detects if nothing to permute", {
    pl <- gl(4, 3)
    n <- 12
    h <- how(within = Within(type = "none"),
             plots = Plots(strata = pl, type = "none"))
    expect_error(check(seq_len(n), control = h),
                 regexp = "Permutation 'type' is \"none\" for both 'plots' & 'within'.\nNothing to permute.")
})

library("testthat")
library("permute")

context("Testing shuffle()")

## test no permutation
test_that("shuffle(n) returns seq_len(n) when not permuting", {
    ctrl <- how(within = Within(type = "none"))

    expect_that(shuffle(3, control = ctrl), is_identical_to(seq_len(3)))
    expect_that(shuffle(1, control = ctrl), is_identical_to(1L))
})

## test shuffle returns integers
test_that("shuffle() returns integers", {
    ctrl <- how(within = Within(type = "none"))

    expect_that(shuffle(4), is_a("integer"))
    expect_that(shuffle(100), is_a("integer"))
    expect_that(shuffle(1, control = ctrl), is_identical_to(1L))
    expect_that(shuffle(3, control = ctrl), is_identical_to(c(1L, 2L, 3L)))
})

## test what shuffle returns when permuting only the strata
## must *not* assume that the samples are in contiguous blocks
test_that("shuffle() works for non-contigous blocks of samples", {
    ## permuting levels of Plots instead of observations
    ## non-contiguous blocks - checks that r1972 continues to work
    Plot <- factor(rep(1:4, 5))
    CTRL <- how(plots = Plots(strata = Plot, type = "free"),
                within = Within(type = "none"))
    n <- 20
    ## FIXME Temporary fix related to #25
    suppressWarnings(RNGversion("3.5.0"))
    set.seed(2)
    result <- shuffle(n, CTRL)
    out1 <- as.integer(c( 3, 2, 1, 4,
                          7, 6, 5, 8,
                         11,10, 9,12,
                         15,14,13,16,
                         19,18,17,20))
    expect_that(result, is_identical_to(out1))
    out2 <- factor(as.integer(rep(c(3,2,1,4), 5)), levels = 1:4)
    expect_that(Plot[result], is_identical_to(out2))
})

test_that("shuffle can permute both plots and within in presence of blocks", {
    ## Example from @LindsayVass on github #9
    control <- how(within = Within(type = "free"),
                   plots = Plots(strata = rep(gl(2,7),2), type = "free"),
                   blocks = gl(2, 14))
    permSet <- shuffle(28, control = control)
    expect_identical(length(permSet), 28L)
    expect_is(permSet, "integer")
})

test_that("constant within plots works", {
    fac <- gl(4, 5)
    ## series permutations
    ctrl <- how(within = Within(type = "series", constant = TRUE),
                plots = Plots(strata = fac, type = "none"))
    perm <- shuffle(length(fac), control = ctrl)
    expect_identical(length(perm), length(fac))
    expect_identical(length(perm), 20L)
    expect_is(perm, "integer")
    ## free/randomisation
    ctrl <- how(within = Within(type = "free", constant = TRUE),
                plots = Plots(strata = fac, type = "none"))
    perm <- shuffle(length(fac), control = ctrl)
    expect_identical(length(perm), length(fac))
    expect_identical(length(perm), 20L)
    expect_is(perm, "integer")
    ## spatial grid 3x3
    fac <- gl(4, 9)
    ctrl <- how(within = Within(type = "grid", nrow = 3, ncol = 3, constant = TRUE),
                plots = Plots(strata = fac, type = "none"))
    perm <- shuffle(length(fac), control = ctrl)
    expect_identical(length(perm), length(fac))
    expect_identical(length(perm), 36L)
    expect_is(perm, "integer")
})

test_that("shuffel works with objects passed to n", {
    obj <- 1:4
    p <- shuffle(obj)
    expect_is(p, "integer")
    expect_identical(length(p), 4L)
    obj <- as.integer(1:4)
    p <- shuffle(obj)
    expect_is(p, "integer")
    expect_identical(length(p), 4L)
    obj <- as.factor(1:4)
    p <- shuffle(obj)
    expect_is(p, "integer")
    expect_identical(length(p), 4L)
    obj <- letters[1:4]
    p <- shuffle(obj)
    expect_is(p, "integer")
    expect_identical(length(p), 4L)
    obj <- matrix(1:16, ncol = 4, nrow = 4)
    p <- shuffle(obj)
    expect_is(p, "integer")
    expect_identical(length(p), 4L)
    obj <- data.frame(A = 1:4, B = letters[1:4])
    p <- shuffle(obj)
    expect_is(p, "integer")
    expect_identical(length(p), 4L)
})

library("testthat")
library("permute")

context("Testing allPerms()")

test_that("allPerms - blocks - within block free", {
    ## example data from Joris Meys from
    ## http://stackoverflow.com/a/21313632/429846
    thedata <- data.frame(score = c(replicate(4, sample(1:3))),
                          judge = rep(1:4, each = 3),
                          wine = rep.int(1:3, 4))

    ## without the observed permutation included
    hh <- how(within = Within("free"),
              blocks = factor(thedata$judge),
              complete = TRUE, maxperm = 1e9)
    nr <- nrow(thedata)
    np <- numPerms(nr, hh)
    p <- allPerms(nr, control = hh)
    expect_that(nrow(p), equals(np - 1)) ## default is to drop observed

    ## check no duplicate indices within rows
    dup <- any(apply(p, 1, function(x) any(duplicated(x))))
    expect_false(dup, info = "Blocks: even; within: free; no observed")

    ## with the observed permutation included
    hh <- how(within = Within("free"),
              blocks = factor(thedata$judge),
              complete = TRUE, maxperm = 1e9,
              observed = TRUE)
    p <- allPerms(nr, control = hh)
    expect_that(nrow(p), equals(np)) ## now includes observed

    ## check no duplicate indices within rows
    dup <- any(apply(p, 1, function(x) any(duplicated(x))))
    expect_false(dup, info = "Blocks: even; within: free; observed")
})

test_that("allPerms; blocks: within; block free - uneven block sizes", {
    fac <- factor(rep(1:3, times = c(2,2,4)))

    ## without the observed permutation included
    hh <- how(within = Within("free"),
              blocks = fac,
              complete = TRUE, maxperm = 1e9)
    ll <- length(fac)
    np <- numPerms(ll, hh)
    expect_that(np, equals(prod(factorial(2), factorial(2), factorial(4))))
    p <- allPerms(ll, control = hh)
    expect_that(nrow(p), equals(np - 1)) ## default is to drop observed

    ## check no duplicate indices within rows
    dup <- any(apply(p, 1, function(x) any(duplicated(x))))
    expect_false(dup, info = "Blocks: uneven; within: free; no observed")

    ## with the observed permutation included
    hh <- how(within = Within("free"),
              blocks = fac,
              complete = TRUE, maxperm = 1e9,
              observed = TRUE)
    p <- allPerms(ll, control = hh)
    expect_that(nrow(p), equals(np)) ## now includes observed

    ## check no duplicate indices within rows
    dup <- any(apply(p, 1, function(x) any(duplicated(x))))
    expect_false(dup, info = "Blocks: uneven; within: free; observed")
})

## testing plot-level permutations ------------------------------------
test_that("allPerms: plots; within: free; even: yes;", {
    fac <- rep(1:3, each = 3)

    hh <- how(plots = Plots(strata = fac),
              complete = TRUE, maxperm = 1e9)
    ll <- length(fac)
    np <- numPerms(ll, hh)
    p <- allPerms(ll, control = hh)
    expect_that(nrow(p), equals(np - 1), ## default is to drop observed
                info = "Check n all perms == numPerms output.")

    ## check no duplicate indices within rows
    dup <- any(apply(p, 1, function(x) any(duplicated(x))))
    expect_false(dup,
                 info = "Unique? Plots: even; within: free; no observed")

    ## with the observed permutation included
    hh <- how(within = Within("free"),
              plot = Plots(strata = fac),
              complete = TRUE, maxperm = 1e9,
              observed = TRUE)
    p <- allPerms(ll, control = hh)
    expect_that(nrow(p), equals(np)) ## now includes observed

    ## check no duplicate indices within rows
    dup <- any(apply(p, 1, function(x) any(duplicated(x))))
    expect_false(dup, info = "Unique? Plots: even; within: free; inc observed")
})

test_that("allPerms; plots: within; plot free - uneven plot sizes", {
    fac <- factor(rep(1:3, times = c(2,2,4)))

    ## without the observed permutation included
    hh <- how(within = Within("free"),
              plots = Plots(strata = fac),
              complete = TRUE, maxperm = 1e9)
    ll <- length(fac)
    np <- numPerms(ll, hh)
    expect_that(np, equals(prod(factorial(2), factorial(2), factorial(4))))
    p <- allPerms(ll, control = hh)
    expect_that(nrow(p), equals(np - 1)) ## default is to drop observed

    ## check no duplicate indices within rows
    dup <- any(apply(p, 1, function(x) any(duplicated(x))))
    expect_false(dup, info = "Plots: uneven; within: free; no observed")

    ## with the observed permutation included
    hh <- how(within = Within("free"),
              plots = Plots(strata = fac),
              complete = TRUE, maxperm = 1e9,
              observed = TRUE)
    p <- allPerms(ll, control = hh)
    expect_that(nrow(p), equals(np)) ## now includes observed

    ## check no duplicate indices within rows
    dup <- any(apply(p, 1, function(x) any(duplicated(x))))
    expect_false(dup, info = "Plots: uneven; within: free; observed")
})

test_that("allPerms; permuting plots only -- non-contiguous plots", {
    transect <- rep(gl(2,2), 2)
    ll <- length(transect)
    ctrl <- how(Within(type = "none"), Plots(type = "free", strata = transect))

    ## without observed
    ref <- matrix(c(3L,4L,1L,2L,7L,8L,5L,6L), nrow = 1, byrow = TRUE)
    perm <- allPerms(ll, ctrl)
    attr(perm, "control") <- NULL
    attr(perm, "observed") <- NULL
    class(perm) <- "matrix"
    expect_that(numPerms(ll, control = ctrl), equals(2L),
                info = "Number of permutations is wrong")
    expect_that(nrow(perm), equals(1L),
                info = "Number of rows in permutation matrix != 1")
    expect_identical(perm, ref)

    ## with observed
    setObserved(ctrl) <- TRUE
    ref <- matrix(c(1L,2L,3L,4L,5L,6L,7L,8L,
                    3L,4L,1L,2L,7L,8L,5L,6L), nrow = 2, byrow = TRUE)
    perm <- allPerms(ll, ctrl)
    perm <- as.matrix(perm)
    expect_that(numPerms(ll, control = ctrl), equals(2L),
                info = "Number of permutations is wrong")
    expect_that(nrow(perm), equals(2L),
                info = "Number of rows in permutation matrix != 2")
    expect_identical(perm, ref,
                     info = "All permutations doesn't match reference")
})

## Grid permutations
test_that("Can generate permutations from a grid design", {
    ## spatial grids within each level of plot, 3 x (4r x 4c)
    nr <- 4
    nc <- 4
    np <- 3 ## number of plots
    plots <- Plots(gl(np, prod(nr, nc)))
    CTRL <- how(plots = plots,
                within = Within(type = "grid", ncol = nc, nrow = nr))
    perms <- allPerms(prod(nr, nc, np), control = CTRL)
    nperms <- numPerms(prod(nr, nc, np), control = CTRL)

    expect_is(perms, "allPerms")
    expect_is(perms, "matrix")
    expect_equal(nperms, nrow(perms) + 1L)

    ## mirroring
    nr <- 3
    nc <- 3
    np <- 2 ## number of plots
    plots <- Plots(gl(np, prod(nr, nc)))
    CTRL <- how(plots = plots,
                within = Within(type = "grid", ncol = nc, nrow = nr,
                                mirror = TRUE))
    perms <- allPerms(prod(nr, nc, np), control = CTRL)
    nperms <- numPerms(prod(nr, nc, np), control = CTRL)

    expect_is(perms, "allPerms")
    expect_is(perms, "matrix")
    expect_equal(nperms, nrow(perms) + 1L)
})

test_that("grids with 2 columns only work correctly", {
    ## spatial grids within each level of plot, (4r x 2c)
    nr <- 4
    nc <- 2
    CTRL <- how(within = Within(type = "grid", ncol = nc, nrow = nr))
    perms <- allPerms(prod(nr, nc), control = CTRL)
    nperms <- numPerms(prod(nr, nc), control = CTRL)

    expect_is(perms, "allPerms")
    expect_is(perms, "matrix")
    expect_equal(nperms, nrow(perms) + 1L)

    ## spatial grids within each level of plot, 3 x (4r x 2c)
    nr <- 4
    nc <- 2
    np <- 3 ## number of plots
    plots <- Plots(gl(np, prod(nr, nc)))
    CTRL <- how(plots = plots,
                within = Within(type = "grid", ncol = nc, nrow = nr))
    perms <- allPerms(prod(nr, nc, np), control = CTRL)
    nperms <- numPerms(prod(nr, nc, np), control = CTRL)

    expect_is(perms, "allPerms")
    expect_is(perms, "matrix")
    expect_equal(nperms, nrow(perms) + 1L)
})


test_that("grids with mirroring & 2 columns only work correctly", {
    ## spatial grids within each level of plot, (4r x 2c)
    nr <- 4
    nc <- 2
    CTRL <- how(within = Within(type = "grid", ncol = nc, nrow = nr,
                                mirror = TRUE))
    perms <- allPerms(prod(nr, nc), control = CTRL)
    nperms <- numPerms(prod(nr, nc), control = CTRL)

    expect_is(perms, "allPerms")
    expect_is(perms, "matrix")
    expect_equal(nperms, nrow(perms) + 1L)

    ## spatial grids within each level of plot, 3 x (4r x 2c)
    nr <- 4
    nc <- 2
    np <- 3 ## number of plots
    plots <- Plots(gl(np, prod(nr, nc)))
    CTRL <- how(plots = plots,
                within = Within(type = "grid", ncol = nc, nrow = nr,
                                mirror = TRUE))
    perms <- allPerms(prod(nr, nc, np), control = CTRL)
    nperms <- numPerms(prod(nr, nc, np), control = CTRL)

    expect_is(perms, "allPerms")
    expect_is(perms, "matrix")
    expect_equal(nperms, nrow(perms) + 1L)
})

test_that("same grid permutation within plots", {
    ## spatial grids within each level of plot, 3 x (4r x 2c)
    nr <- 4
    nc <- 2
    np <- 3 ## number of plots
    plots <- Plots(gl(np, prod(nr, nc)))
    CTRL <- how(plots = plots,
                within = Within(type = "grid", ncol = nc, nrow = nr,
                                constant = TRUE))
    perms <- allPerms(prod(nr, nc, np), control = CTRL)
    nperms <- numPerms(prod(nr, nc, np), control = CTRL)

    expect_is(perms, "allPerms")
    expect_is(perms, "matrix")
    expect_equal(nperms, nrow(perms) + 1L)
})

test_that("same grid permutation within plots & mirroring", {
    ## spatial grids within each level of plot, 3 x (4r x 2c)
    nr <- 4
    nc <- 2
    np <- 3 ## number of plots
    plots <- Plots(gl(np, prod(nr, nc)))
    CTRL <- how(plots = plots,
                within = Within(type = "grid", ncol = nc, nrow = nr,
                                constant = TRUE, mirror = TRUE))
    perms <- allPerms(prod(nr, nc, np), control = CTRL)
    nperms <- numPerms(prod(nr, nc, np), control = CTRL)

    expect_is(perms, "allPerms")
    expect_is(perms, "matrix")
    expect_equal(nperms, nrow(perms) + 1L)
})

test_that("allPerms works with complex, but small, design", {
    h <- how(within = Within(type = "series", constant = TRUE),
             plots = Plots(strata = gl(2, 5), type = "series", mirror = TRUE))
    ap <- allPerms(10, control = h)
    expect_is(ap, "matrix")
    expect_equal(nrow(ap), 10 - 1L)
})

test_that("summary.allPerms works & prints correctly", {
    a <- c("Ar","Ba","Bl","Bu","Ca")
    ap <- allPerms(a)
    ## FIXME: this doesn't work yet in released testthat
    ## expect_output(print(ap))
    expect_output(print(summary(ap)),
                  regexp = "Complete enumeration of permutations")
})

# issue #28
test_that("allPerms works for series within blocks issue 28", {
    ## Set up factors for the Plots and Blocks
    plts <- gl(4, 10) ## 4 Plots of 10 samples each
    blks <- gl(2, 20) ## 2 Blocks of 20 samples each
    ## define permutation design
    h1 <- how(Within(type = "series"), blocks = blks)
    p <- allPerms(40, h1)
    expect_equal(numPerms(40, control = h1), 400L)
    expect_identical(nrow(p), 399L)
})

`print.how` <- function(x, ...) {

    ## only for objects of correct class
    stopifnot(class(x) == "how")

    ## prefix to add to sub-elements
    pfix <- "  "

    cat("\n")
    writeLines(strwrap("Permutation Design:"))
    cat("\n")

    ## Blocks
    writeLines("Blocks:")
    blocks <- getBlocks(x)
    if (is.null(blocks)) {
        writeLines(strwrap("Defined by: none", prefix = pfix))
    } else {
        writeLines(strwrap(paste("Blocks:", x$blocks.name),
                           prefix = pfix))
    }

    cat("\n")

    ## Plots
    writeLines("Plots:")
    plotStr <- getStrata(x, which = "plots")
    plots <- getPlots(x)
    ptype <- getType(x, which = "plots")
    if (is.null(plotStr)) {
        writeLines(strwrap("Defined by: none", prefix = pfix))
    } else {
        writeLines(strwrap(paste("Plots:", plots$plots.name),
                           prefix = pfix))
        writeLines(strwrap(paste("Permutation type:", ptype),
                           prefix = pfix))
        mirrorP <- getMirror(x, which = "plots")
        writeLines(strwrap(paste("Mirrored?:", if(mirrorP) "Yes" else "No"),
                           prefix = pfix))
        if(isTRUE(all.equal(ptype, "grid"))) {
            nr <- getRow(x, which = "plots")
            nr.t <- if(nr > 1) "rows" else "row"
            nc <- getCol(x, which = "plots")
            nc.t <- if(nc > 1) "cols" else "col"
            writeLines(strwrap(paste("Grid dimensions:", nr, nr.t, " ",
                                     nc, nc.t),
                               prefix = pfix))
        }
    }

    cat("\n")

    ## Within plots
    writeLines("Within Plots:")
    wtype <- getType(x, which = "within")
    writeLines(strwrap(paste("Permutation type:", wtype), prefix = pfix))
    mirrorW <- getMirror(x, which = "within")
    constantW <- getConstant(x)
    txt <- "Different permutation within each Plot?:"
    if(isTRUE(wtype %in% c("series", "grid"))) {
        writeLines(strwrap(paste("Mirrored?:", if(mirrorW) "Yes" else "No"),
                           prefix = pfix))
        writeLines(strwrap(paste(txt, if(constantW) "No" else "Yes"),
                           prefix = pfix))
    }
    if(isTRUE(all.equal(wtype, "grid"))) {
        nr <- getRow(x, which = "within")
        nr.t <- if(nr > 1) "rows" else "row"
        nc <- getCol(x, which = "within")
        nc.t <- if(nc > 1) "cols" else "col"
        writeLines(strwrap(paste("Grid dimensions:", nr, nr.t, " ",
                                 nc, nc.t),
                           prefix = pfix))
    }

    cat("\n")

    ## Meta data
    writeLines("Permutation details:")
    writeLines(strwrap(paste("Number of permutations:",
                             getNperm(x)), prefix = pfix))
    writeLines(strwrap(paste("Max. number of permutations allowed:",
                             getMaxperm(x)), prefix = pfix))
    txt <- paste("Evaluate all permutations?:",
                 if(getComplete(x)) "Yes." else "No.",
                 "  Activation limit:", getMinperm(x))
    writeLines(strwrap(txt, prefix = pfix))

}

`allPerms` <- function(n, control = how(), check = TRUE) {
    ## start
    v <- n
    ## expand n if a numeric or integer vector of length 1
    if((is.numeric(n) || is.integer(n)) && (length(n) == 1))
         v <- seq_len(n)
    ## number of observations in data
    n <- nobs(v)
    ## check permutation scheme and update control
    make <- getMake(control)
    if (check) {
        control2 <- control
        setMake(control2) <- FALSE
        pcheck <- check(v, control = control2, quietly = TRUE)
    }
    ## ctrl <- pcheck$control
    ## if we do copy the new updated control, we need to update to
    ## reset make
    ## ctrl <- update(ctrl, make = make)

    ## get max number of permutations
    nperms <- numPerms(v, control = control)

    ## sanity check - don't let this run away to infinity
    ## esp with type = "free"
    if(nperms > getMaxperm(control))
        stop("Number of possible permutations too large (> 'maxperm')")

    WI <- getWithin(control)
    strataP <- getStrata(control, which = "plots")
    typeW <- getType(control, which = "within")
    typeP <- getType(control, which = "plot")
    BLOCKS <- getBlocks(control)
    dimW <- getDim(control, which = "within")
    dimP <- getDim(control, which = "plots")
    mirrorW <- getMirror(control, which = "within")
    mirrorP <- getMirror(control, which = "plots")
    constantW <- getConstant(control)

    ## give a BLOCKS if non supplied - i.e. one block
    if(is.null(BLOCKS))
        BLOCKS <- factor(rep(1, n))

    ## split v by blocks
    spl <- split(seq_len(n), BLOCKS)
    nb <- length(spl) # number of blocks

    ## result object
    out <- vector(mode = "list", length = nb)

    ## null-out Blocks in control
    control2 <- control
    setBlocks(control2) <- NULL

    ## loop over blocks and return allPerms on each block
    for (i in seq_along(spl)) {
        out[[i]] <-
            doAllPerms(spl[[i]], strataP, typeW, typeP, mirrorW,
                       mirrorP, constantW, dimW, dimP, control2,
                       nperms = nperms)
    }

    ## bind all blocks together, repeating them as required
    out <- cbindAllPerms(out)

    ## bind all the blocks together
    ## out <- do.call(cbind, out) ## hmm are any of these the same shape?
    out[, unlist(spl)] <- out  ## is this being done at the doAllPerms level?

    if(!(observed <- getObserved(control))) {
        obs.v <- seq_len(n)
        obs.row <- apply(out, 1, function(x, obs.v) all(x == obs.v), obs.v)
        out <- out[!obs.row, , drop = FALSE]
        ## reduce the number of permutations to get rid of the
        ## observed ordering
        setNperm(control) <- getNperm(control) - 1
    }

    ## as a permutationMatrix we pick up nice print method
    class(out) <- c("allPerms", "permutationMatrix", "matrix")
    attr(out, "control") <- control
    attr(out, "observed") <- observed
    out
}


`doAllPerms` <- function(obs, strataP, typeW, typeP, mirrorW, mirrorP,
                         constantW, dimW, dimP, control, nperms) {
    n <- length(obs)

    ## subset strataP to take only the obs indices and drop the unused
    ## levels
    if (!is.null(strataP)) {
        strataP <- droplevels(strataP[obs])
    }

    ## also need to update the $strata component of control
    ## FIXME: this really should have a toplevel function to set/update
    ## sub-components of control
    ## Pl <- getPlots(control)
    ## setStrata(Pl) <- strataP
    ## setPlots(control) <- Pl
    control$plots$strata <- strataP

    ## permuting within?
    if (typeW != "none") {
        if(is.null(strataP)) {
            ## no plot-level permutations
            ## have to redo numPerms here because we could be within a block
            res <- switch(typeW,
                          free = allFree(n),
                          series = allSeries(n, numPerms(n, control), mirrorW),
                          grid = allGrid(n, numPerms(n, control),
                                         dimW[1], dimW[2],
                                         mirrorW, constantW))
            ## use res to index original observation indices in this group
            res[] <- obs[res]
        } else {
            ## permuting within plots
            tab <- table(strataP)
            pg <- unique(tab)
            ng <-  length(tab)
            if(constantW) {
                ## same permutation in each plot
                controlW <- how(within = getWithin(control))
                nperms <- numPerms(pg, controlW)
                ord <- switch(typeW,
                              free = allFree(pg),
                              series = allSeries(pg, nperms, mirrorW),
                              grid = allGrid(pg, nperms, dimW[1],
                              dimW[2], mirrorW, constantW))
                res <- vector(mode = "list", length = ng)
                ss <- seq(0, to = prod(pg, ng-1), by = pg)
                for (i in seq_len(ng)) {
                    res[[i]] <- ord + ss[i]
                }
                ## same permutation within plots, so just cbind rather than
                ## cbindAllPerms as we don't need all combns of rows
                res <- do.call(cbind, res)
                res[] <- obs[res] ## index into the observations in this block
            } else {
                ## different permutations within plots
                nperms <- numPerms(sum(tab), control)

                if(length(pg) > 1) {
                    ## different number of observations per level of strata
                    if(typeW == "grid")
                        ## FIXME: this should not be needed once all checks are
                        ## in place in check()
                        stop("Unbalanced grid designs are not supported")
                    controlW <- how(within = getWithin(control))
                    res <- vector(mode = "list", length = ng)
                    add <- c(0, cumsum(tab)[1:(ng-1)])
                    for(j in seq_along(tab)) {
                        np <- numPerms(tab[j], controlW)
                        ord <- switch(typeW,
                                      free = allFree(tab[j]),
                                      series = allSeries(tab[j], np, mirrorW))
                        res[[j]] <- ord + add[j]
                    }
                    res <- cbindAllPerms(res)
                    res[] <- obs[res]
                } else {
                    ## same number of observations per level of strata
                    controlW <- how(within = getWithin(control))
                    np <- numPerms(pg, controlW)
                    ord <-
                        switch(typeW,
                               free = allFree(pg),
                               series = allSeries(pg, np, mirrorW),
                               grid = allGrid(pg, np, dimW[1],
                               dimW[2], mirrorW, constantW))
                    res <- vector(mode = "list", length = ng)
                    ss <- seq(0, to = prod(pg, ng-1), by = pg)
                    for(i in seq_len(ng)) {
                        res[[i]] <- ord + ss[i]
                    }
                    res <- cbindAllPerms(res)
                    res[] <- obs[res]
                }
            }
        }
    }
    ## Do we need to permute plots?
    if (!is.null(strataP) && !isTRUE(all.equal(typeP, "none"))) {
        ## permuting plots ONLY
        if(typeW == "none") {
            res <- allStrata(n, control = control)
        } else {
            ## FIXME - this need updating to work with the new code
            ## permuting blocks AND within blocks
            ## need a local CONTROL that just permutes blocks
            controlP <- how(plots = Plots(strata = strataP, type = typeP),
                            within = Within(type = "none", constant = constantW))
            ## FIXME - the above should really only need to update
            ## within as shown, not fiddle with Plots

            ## number of permutations for just the block level
            permP <- numPerms(n, control = controlP)
            ## get all permutations for the block level
            shuffP <- allStrata(n, control = controlP)
            ## copy the set of permutations for within blocks
            ## permP times - results is a list
            resP <- rep(list(res), permP)
            resP <- lapply(seq_along(resP),
                            function(k, wi, bl) {
                                t(apply(wi[[k]], 1,
                                        function(x, bl, kk) {
                                            x[bl[kk,]]
                                        }, bl = bl, kk = k))
                            },
                            wi = resP, bl = shuffP)
            res <- do.call(rbind, resP)
        }
    }
    ## some times storage.mode of res is numeric, sometimes
    ## it is integer, set to "integer" for comparisons using
    ## identical to match the observed ordering
    storage.mode(res) <- "integer"

    ## return
    res
}

## enumerate all possible permutations for a more complicated
## design
## fac <- gl(2,6)
##ctrl <- how(type = "grid", mirror = FALSE, strata = fac,
##                    constant = TRUE, nrow = 3, ncol = 2)
## ctrl <- how(strata = fac,
##                     within = Within(type = "grid", mirror = FALSE,
##                     constant = TRUE, nrow = 3, ncol = 2),
##                     blocks = Blocks(type = "free"))
## Nobs <- length(fac)
## numPerms(seq_len(Nobs), control = ctrl)
## numPerms(Nobs, control = ctrl) ## works just as well
## (tmp <- allPerms(Nobs, control = ctrl, observed = TRUE))
## (tmp2 <- allPerms(Nobs, control = ctrl))

`as.allPerms` <- function(object, control) {
    object <- as.matrix(object)
    class(object) <- c("allPerms", "matrix")
    if (!missing(control)) {
        attr(object, "control") <- control
        attr(object, "observed") <- getObserved(control)
    } else {
        ## This needs double checking that it is needed!
        attr(object, "control") <- attr(object, "observed") <- NA
    }
    object
}

## nobs generic is defined only in R 2.13.0: define here for older R
if (getRversion() < "2.13.0")
    nobs <- function(object, ...) UseMethod("nobs")

## add some nobs() methods - need to be documented
`nobs.numeric` <- function(object, ...) {
    length(object)
}

`nobs.integer` <- function(object, ...) {
    nobs.numeric(object, ...)
}

`nobs.matrix` <- function(object, ...) {
    NROW(object)
}

`nobs.data.frame` <- function(object, ...) {
    NROW(object)
}

`nobs.factor` <- function(object, ...) {
    length(object)
}

`nobs.character` <- function(object, ...) {
    length(object)
}

library("testthat")
library("permute")

context("Testing nobs() methods")

test_that("numeric nobs method", {
    v <- runif(10)
    n <- nobs(v)
    expect_identical(n, 10L)
})

test_that("integer nobs method", {
    v <- 1L:10L
    n <- nobs(v)
    expect_identical(n, 10L)
})

test_that("matrix nobs method", {
    m <- matrix(1:9, nrow = 3)
    n <- nobs(m)
    expect_identical(n, 3L)
})

test_that("data frame nobs method", {
    df <- as.data.frame(matrix(1:9, nrow = 3))
    n <- nobs(df)
    expect_identical(n, 3L)
})

test_that("factor nobs method", {
    f <- factor(c(1,2,3,2,1,4,5,2,1,4))
    n <- nobs(f)
    expect_identical(n, 10L)
})

test_that("character nobs method works", {
    ch <- c("Ar","Ba","Bl","Bu","Ca")
    n <- nobs(ch)
    expect_identical(n, 5L)
    expect_identical(n, length(ch))
})

`Plots` <- function(strata = NULL, type = c("none","free","series","grid"),
                    mirror = FALSE, ncol = NULL, nrow = NULL) {

    plots.name <- deparse(substitute(strata))
    ## strata should also be a factor - coerce
    if(!is.null(strata))
        strata <- as.factor(strata)

    type <- match.arg(type)

    ## process the call to make it standalone
    .call <- match.call()
    if (length(.call) > 1L) {
        .ll <- as.list(.call[-1])
        for (i in seq_along(.ll))
            .ll[[i]] <- eval(.ll[[i]], parent.frame())
        .ll <- c(as.list(.call[[1]]), .ll)
        names(.ll) <- names(.call)
        .call <- as.call(.ll)
    }

    out <- list(strata = strata, type = type, mirror = mirror,
                ncol = ncol, nrow = nrow,
                plots.name = plots.name, call = .call)
    class(out) <- "Plots"
    out
}

library("testthat")
library("permute")

context("Testing how()")

test_that("how() works with explicit NULL blocks arg", {
    ## Example of failure from Jari github #8
    h <- how(blocks = NULL)
    expect_that(h, is_a("how"))
})

test_that("print method for how", {
    expect_output(print(how()), regexp = "Permutation Design:")

    ctrl <- how(plots = Plots(strata = gl(4,5)))
    expect_output(print(how()), regexp = "Plots:")

    ctrl <- how(plots = Plots(strata = gl(4,9), type = "grid", ncol = 3, nrow = 3))
    expect_output(print(ctrl), regexp = "Grid dimensions:")
})

`numPerms` <- function(object, control = how()) {
  ## constant holding types where something is permuted
  TYPES <- c("free","grid","series","none")

  ## expand object if a numeric or integer vector of length 1
  if((is.numeric(object) || is.integer(object)) &&
     (length(object) == 1))
    object <- seq_len(object)
  ## number of observations in data
  n <- nobs(object)

  ## get the permutation levels from control
  WI <- getWithin(control)
  PL <- getPlots(control)
  BL <- getBlocks(control)

  ## any strata to permute within / blocking?
  BLOCKS <- getStrata(control, which = "blocks")
  PSTRATA <- getStrata(control, which = "plots")
  typeP <- getType(control, which = "plots")
  typeW <- getType(control, which = "within")

  ## mirroring?
  mirrorP <- getMirror(control, which = "plots")
  mirrorW <- getMirror(control, which = "within")

  ## constant - i.e. same perm within each plot?
  constantW <- getConstant(control)

  ## grid dimensions
  colW <- getCol(control, which = "within")
  colP <- getRow(control, which = "plots")

  ## Some checks; i) Plot strata must be of same size when permuting strata
  ##                 or having the same constant permutation within strata
  ##             ii) In grid designs, grids must be of the same size for all
  ##                 strata
  ##
  ## FIXME - this probably should be in check()!
  if(!is.null(PSTRATA)) {
    tab <- table(PSTRATA)
    same.n <- length(unique(tab))
    if((typeP != "none" || isTRUE(constantW)) && same.n > 1) {
      stop("All levels of strata must have same number of samples for chosen scheme")
    }
    if(typeP == "grid" && same.n > 1) {
      stop("Unbalanced grid designs are not supported")
    }
  }

  ## the various designs allowed imply multipliers to number of samples
  ## for the restricted permutations

  mult.p <- mult.wi <- 1

  ## within types
  if(typeW %in% c("series","grid")) {
    mult.wi <- 2
    if(isTRUE(all.equal(typeW, "grid")) && !is.null(colW) && colW > 2) {
      mult.wi <- 4
    } else {
      if(isTRUE(all.equal(n, 2)))
        mult.wi <- 1
    }
  }
  ## plot-level types
  if(typeP %in% c("series","grid")) {
    mult.p <- 2
    if(isTRUE(all.equal(typeP, "grid")) && !is.null(colP) && colP > 2) {
      mult.p <- 4
    } else {
      if(isTRUE(all.equal(length(tab), 2))) # was all.equal(n, 2)
        mult.p <- 1
    }
  }

  ## within
  ## another check - shouldn't this be moved? FIXME
  if(!typeW %in% TYPES) {
    stop("Ambiguous permutation type in 'control$within$type'")
  }

  ## calculate the number of possible permutations

  ## Compute number of permutations for each block
  if(is.null(BLOCKS))
      BLOCKS <- factor(rep(1, n))

  ## split an index vector
  indv <- seq_len(n)
  spl <- split(indv, BLOCKS)

  ## loop over the components of spl & apply doNumPerms
  np <- sapply(spl, doNumPerms, mult.p, mult.wi, typeP, typeW, PSTRATA,
               mirrorP, mirrorW, constantW)

  ## multiply up n perms per block
  round(prod(np), 0)
}

`doNumPerms` <- function(obs, mult.p, mult.wi, typeP, typeW, PSTRATA,
                         mirrorP, mirrorW, constantW) {
    n <- nobs(obs) ## obs is index vector for object, split by blocks

    if(!is.null(PSTRATA)) {
        ## take only the PSTRATA needed for this block, drop unused levels
        PSTRATA <- droplevels(PSTRATA[obs])

        ## need only those strata for the current block. As obs is the index
        ## vector, split by block, this now gives nobs per plot strata
        tab <- table(PSTRATA)
        same.n <- length(unitab <- unique(tab))
    }

    ## plots
    num.p <- if(isTRUE(all.equal(typeP, "free"))) {
        exp(lfactorial(length(levels(PSTRATA))))
    } else if(typeP %in% c("series", "grid")) {
        if(isTRUE(mirrorP)) {
            mult.p * length(tab)
        } else {
            length(tab)
        }
    } else {
        1
    }

    num.wi <- if(isTRUE(all.equal(typeW, "none"))) {
        ## no within permutations. note we multiply num.p by this
        ## values so it is 1 not 0!!
        1
    } else if(isTRUE(all.equal(typeW, "free"))) {
        if(!is.null(PSTRATA)) {
            if(constantW) {
                factorial(tab[1])
            } else {
                prod(factorial(tab))
            }
        } else {
            exp(lfactorial(n))
        }
    } else {
        if(!is.null(PSTRATA)) {
            if(same.n > 1) {
                multi <- rep(2, length = length(tab))
                multi[which(tab == 2)] <- 1
                if(mirrorW) {
                    prod(multi * tab)
                } else {
                    prod(tab)
                }
            } else {
                if(mirrorW) {
                    if(constantW) {
                        mult.wi * unitab[1]
                    } else {
                        prod(mult.wi * tab)
                    }
                } else {
                    if(constantW) {
                        unitab[1] ## FIXME: unitab[1]?? (unique(tab)[1])
                    } else {
                        prod(tab)
                    }
                }
            }
        } else {
            if(mirrorW)
                mult.wi * n
            else
                n
        }
    }

    ## return, trying to avoid floating point issues
    zapsmall(num.p) * zapsmall(num.wi)
}

`permute` <- function(i, n, control) {
    complete <- getComplete(control)
    ap <- getAllperms(control)
    perm <- if (complete && !is.null(ap)) {
        ap[i, ]                 # select ith permutation
    } else {
        if (complete) {
            warning("'$all.perms' is NULL, yet '$complete = TRUE'.\nReturning a random permutation.")
        }
        shuffle(n, control)
    }
    perm
}

library("testthat")
library("permute")

context("Testing permute() function")

test_that("permute() returns ith row of all perms", {
    h <- how()
    v <- 1:4
    ch <- check(v, h, quietly = TRUE)
    h <- ch$control
    p1 <- permute(1, n = length(v), control = h)
    expect_equal(length(p1), 4L)
    expect_is(p1, "integer")
    expect_equal(p1, c(1L, 2L, 4L, 3L))
    expect_equal(p1, getAllperms(h)[1, ])

    p21 <- permute(21, n = length(v), control = h)
    expect_equal(length(p21), 4L)
    expect_is(p21, "integer")
    expect_equal(p21, c(4L, 2L, 3L, 1L))
    expect_equal(p21, getAllperms(h)[21, ])
})

test_that("permute() returns a random permutation if no $allperms", {
    h <- how()
    v <- 1:10                           # want something big so no allperms
    p <- permute(10, n = length(v), control = h)
    expect_equal(length(p), length(v))
    expect_true(all(p >= 1))
    expect_true(all(p <= 10))

    setComplete(h) <- TRUE
    expect_warning(permute(10, n = length(v), control = h),
                   regexp = "Returning a random permutation")
})

`check` <- function(object, control = how(), quietly = FALSE)
{
    ## In pricinple we are mainly dealing with integers, but many
    ## functions do not return integers but double, and the numbers
    ## can be so large that they overflow integer and they really must be
    ## double. Therefore we define EPS as a nice value between two
    ## successive integers
    EPS <- 0.5
    ## if object is numeric or integer and of length 1,
    ## extend the object
    if(length(object) == 1 &&
       (is.integer(object) || is.numeric(object)))
        object <- seq_len(object)

    ## check the number of observations in object
    N <- nobs(object)

    ## sample permutation type
    typeW <- getType(control, which = "within")
    typeP <- getType(control, which = "plots")

    ## check we're actually permuting something
    if (identical(typeW, typeP) && isTRUE(all.equal(typeW, "none"))) {
        stop("Permutation 'type' is \"none\" for both 'plots' & 'within'.\nNothing to permute.")
    }

    ## strata at plot & block levels
    plots <- getStrata(control, which = "plots")
    blocks <- getStrata(control, which = "blocks")

    ## if strata, check N == length of strata but beware empty levels
    if(!is.null(plots)) {
        tab <- table(plots)
        if(!identical(as.integer(N), as.integer(sum(tab))))
            stop("Number of observations and length of Plot 'strata' do not match.")

        ## if "grid", check design balanced?
        if((bal <- length(unique(tab))) > 1 && typeW == "grid")
            stop("Unbalanced 'grid' designs are not supported.")

        ## if grid design, check nrow*ncol is multiple of N
        if(typeW == "grid" &&
           !identical(N %% prod(getDim(control, which = "within")), 0))
            stop("Within 'nrow' * 'ncol' not a multiple of number of observations.")

        ## if constant, check design balanced?
        if(getConstant(control) && bal > 1)
            stop("Unbalanced designs not allowed with 'constant = TRUE'.")

        ## if permuting strata, must be balanced
        if(typeP != "none" && bal > 1)
            stop("Design must be balanced if permuting 'strata'.")

        ## if permuting Plots as a grid check dimensions match levels of
        ## Plot-level strata
        if(isTRUE(all.equal(typeP, "grid"))) {
            levP <- levels(Plots)
            dimP <- getDim(control, which = "plots")
            if(!identical(levP, prod(dimP))) {
                stop("Plot 'nrow' * 'ncol' not a multiple of number of Plots.")
            }
        }
    }

    ## check length of Blocks is equal to N
    if(!is.null(blocks)) {
        if(!isTRUE(all.equal(length(blocks), N)))
            stop("Number of observations and length of Block 'strata' do not match.")
    }

    ## check allPerms is of correct form
    if(!is.null(control$all.perms) &&
       !inherits(control$all.perms, "allPerms"))
        stop("'control$all.perms' must be of class 'allPerms'.")

    ## get number of possible permutations
    num.pos <- numPerms(object, control)

    ## check if number requested permutations exceeds or equals max
    ## possible
    nperm <- getNperm(control)
    if(nperm + EPS > (num.pos - !getObserved(control))) {
        setComplete(control) <- TRUE
        setMaxperm(control) <- num.pos
        setNperm(control) <- num.pos - !getObserved(control)
        if(!quietly)
            message("'nperm' >= set of all permutations: complete enumeration.")
    }

    ## if number of possible perms < minperm turn on complete
    ## enumeration
    if((num.pos - !getObserved(control)) < getMinperm(control) + EPS) {
        setComplete(control) <- TRUE
        setMaxperm(control) <- num.pos
        if(!quietly)
            message("Set of permutations < 'minperm'. Generating entire set.")
    }

    ## if complete enumeration, generate all permutations
    if(getComplete(control) && getMake(control)) {
        ap <- allPerms(N, control = control, check = FALSE)
        setAllperms(control) <- ap
        setNperm(control) <- nrow(ap)
    }
    retval <- list(n = num.pos, control = control)
    class(retval) <- "check"
    retval
}

## Modified version of allFree() provided by Doug Bates
## via personal email on 19 Jan 2012
`allFree` <- function(n, v = seq_len(n)) {
    if(n == 1L) return(array(v, c(1L, 1L)))
    do.call(rbind,
            lapply(seq_len(n),
                   function(i) cbind(v[i], allFree(n - 1L, v[-i]))))
}

`allGrid` <- function(n, nperms, nr, nc, mirror, constant)
{
    v <- seq_len(n)
    X <- matrix(nrow = nperms, ncol = n)
    idx <- 1
    ## ncol == 2 is special case
    if(nc == 2) {
        X <- allSeries(n, nperms = nperms, mirror = mirror)
    } else {
        for(i in seq_len(nr)) {
            for(j in seq_len(nc)) {
                ir <- seq(i, length = nr)%%nr
                ic <- seq(j, length = nc)%%nc
                ## block 1 - no reversals
                X[idx, ] <- rep(ic, each = nr) * nr +
                    rep(ir, len = nr * nc) + 1
                if(mirror) {
                    ## block 2 - rev rows but not columns
                    X[idx + n, ] <- rep(ic, each = nr) * nr +
                        rep(rev(ir), len = nr * nc) + 1
                    ## block 3 - rev columns but not rows
                    X[idx + (2*n), ] <- rep(rev(ic), each = nr) *
                        nr + rep(ir, len = nr * nc) + 1
                }
                idx <- idx + 1
            }
        }
        if(mirror) {
            ## rev columns and rows
            ## no calculations, just rev cols of block 1
            v <- seq_len(n)
            X[((3*n)+1):(4*n), ] <- X[v, rev(v)]
        }
    }
    X
}

## Extractor functions for blocks, plots and within, plus strata,
## etc ...

## Blocks
`getBlocks` <- function(object, ...) {
    UseMethod("getBlocks")
}

`getBlocks.default` <- function(object, ...) {
    stop("No default method for 'getBlocks()'")
}

`getBlocks.how` <- function(object, ...) {
    object$blocks
}

## Plots
`getPlots` <- function(object, ...) {
    UseMethod("getPlots")
}

`getPlots.default` <- function(object, ...) {
    stop("No default method for 'getPlots()'")
}

`getPlots.how` <- function(object, ...) {
    object$plots
}

## Within plots
`getWithin` <- function(object, ...) {
    UseMethod("getWithin")
}

`getWithin.default` <- function(object, ...) {
    stop("No default method for 'getWithin()'")
}

`getWithin.how` <- function(object, ...) {
    object$within
}

## Strata
`getStrata` <- function(object, ...) {
    UseMethod("getStrata")
}

`getStrata.default` <- function(object, ...) {
    stop("No default method for 'getStrata()'")
}

`getStrata.how` <- function(object,
                                  which = c("plots","blocks"),
                                  drop = TRUE, ...) {
    which <- match.arg(which)
    if(isTRUE(all.equal(which, "plots")))
        strata <- object$plots$strata
    else if(isTRUE(all.equal(which, "blocks")))
        strata <- object$blocks #object$blocks$strata
    else
        stop("Ambiguous `which`")
    if(isTRUE(drop) && !is.null(strata))
        strata <- droplevels(strata)
    strata
}

`getStrata.Plots` <- function(object, drop = TRUE, ... ) {
    strata <- object$strata
    if(isTRUE(drop) && !is.null(strata))
        strata <- droplevels(strata)
    strata
}

## Get type of permutation
`getType` <- function(object, ...) {
    UseMethod("getType")
}

`getType.default` <- function(object, ...) {
    stop("No default method for 'getType()'")
}

`getType.how` <- function(object,
                          which = c("plots","within"), ...) {
    which <- match.arg(which)
  if(isTRUE(all.equal(which, "plots")))
      type <- getPlots(object)$type
  else if(isTRUE(all.equal(which, "within")))
      type <- getWithin(object)$type
  else
      stop("Ambiguous `which`")
  type
}

`getType.Within` <- function(object, ...) {
    object$type
}

`getType.Plots` <- function(object, ...) {
    object$type
}

## suppose we can also have setBlocks() etc...
## to update the control object in place....

## Get mirroring status
`getMirror` <- function(object, ...) {
    UseMethod("getMirror")
}

`getMirror.default` <- function(object, ...) {
    stop("No default method for 'getMirror()'")
}

`getMirror.how` <- function(object,
                                    which = c("plots","within"), ...) {
    which <- match.arg(which)
    if(isTRUE(all.equal(which, "plots")))
        mirror <- getPlots(object)$mirror
    else if(isTRUE(all.equal(which, "within")))
        mirror <- getWithin(object)$mirror
    else
        stop("Ambiguous `which`")
    mirror
}

`getMirror.Within` <- function(object, ...) {
    object$mirror
}

`getMirror.Plots` <- function(object, ...) {
    object$mirror
}

## Get constant status - i.e. same permutation in each Plot
`getConstant` <- function(object, ...) {
    UseMethod("getConstant")
}

`getConstant.default` <- function(object, ...) {
    stop("No default method for 'getConstant()'")
}

`getConstant.how` <- function(object, ...) {
    getWithin(object)$constant
}

`getConstant.Within` <- function(object, ...) {
    object$constant
}

## Get the number of rows and colums from grid designs
`getRow` <- function(object, ...) {
    UseMethod("getRow")
}

`getRow.default` <- function(object, ...) {
    NROW(object)
}

`getRow.how` <- function(object, which = c("plots","within"),
                                 ...) {
    which <- match.arg(which)
    if(isTRUE(all.equal(which, "plots")))
        nrow <- getPlots(object)$nrow
    else if(isTRUE(all.equal(which, "within")))
        nrow <- getWithin(object)$nrow
    else
        stop("Ambiguous `which`")
    nrow
}

`getRow.Within` <- function(object, ...) {
    object$nrow
}

`getRow.Plots` <- function(object, ...) {
    object$nrow
}

`getCol` <- function(object, ...) {
    UseMethod("getCol")
}

`getCol.default` <- function(object, ...) {
    NCOL(object)
}

`getCol.how` <- function(object, which = c("plots","within"),
                                 ...) {
    which <- match.arg(which)
    if(isTRUE(all.equal(which, "plots")))
        ncol <- getPlots(object)$ncol
    else if(isTRUE(all.equal(which, "within")))
        ncol <- getWithin(object)$ncol
    else
        stop("Ambiguous `which`")
    ncol
}

`getCol.Within` <- function(object, ...) {
    object$ncol
}

`getCol.Plots` <- function(object, ...) {
    object$ncol
}

`getDim` <- function(object, ...) {
    UseMethod("getDim")
}

`getDim.default` <- function(object, ...) {
    dim(object)
}

`getDim.how` <- function(object, which = c("plots","within"),
                                 ...) {
    which <- match.arg(which)
    if(isTRUE(all.equal(which, "plots"))) {
        PL <- getPlots(object)
        nc <- PL$ncol
        nr <- PL$nrow
    } else if(isTRUE(all.equal(which, "within"))) {
        WI <- getWithin(object)
        nc <- WI$ncol
        nr <- WI$nrow
    } else {
        stop("Ambiguous `which`")
    }
    c(nr, nc)
}

`getDim.Within` <- function(object, ...) {
    c(object$nrow, object$ncol)
}

`getDim.Plots` <- function(object, ...) {
    c(object$nrow, object$ncol)
}

## return the requested number of permutations
`getNperm` <- function(object, ...) {
    UseMethod("getNperm")
}

`getNperm.default` <- function(object, ...) {
    stop("No default method for `getNperm`")
}

`getNperm.how` <- function(object, ...) {
    object$nperm
}

## Returns maximum permutation threshold
`getMaxperm` <- function(object, ...) {
    UseMethod("getMaxperm")
}

`getMaxperm.default` <- function(object, ...) {
    stop("No default method for `getMaxperm`")
}

`getMaxperm.how` <- function(object, ...) {
    object$maxperm
}

## Returns minimum permutation threshold
`getMinperm` <- function(object, ...) {
    UseMethod("getMinperm")
}

`getMinperm.default` <- function(object, ...) {
    stop("No default method for `getMinperm`")
}

`getMinperm.how` <- function(object, ...) {
    object$minperm
}

## Returns status of complete enumeration
`getComplete` <- function(object, ...) {
    UseMethod("getComplete")
}

`getComplete.default` <- function(object, ...) {
    stop("No default method for `getComplete`")
}

`getComplete.how` <- function(object, ...) {
    object$complete
}

## Returns whether all permutation should/should not be made
`getMake` <- function(object, ...) {
    UseMethod("getMake")
}

`getMake.default` <- function(object, ...) {
    stop("No default method for `getMake`")
}

`getMake.how` <- function(object, ...) {
    object$make
}

## Returns whether the observed permutation should be in
## the set of permutations
`getObserved` <- function(object, ...) {
    UseMethod("getObserved")
}

`getObserved.default` <- function(object, ...) {
    stop("No default method for `getObserved`")
}

`getObserved.how` <- function(object, ...) {
    object$observed
}

## Extractor for all.perms component
`getAllperms` <- function(object, ...) {
    UseMethod("getAllperms")
}

`getAllperms.how` <- function(object, ...) {
    object$all.perms
}

`getAllperms.default` <- function(object, ...) {
    stop("No default method for `getAllperms`")
}

## Extractor for control/how objects
`getControl` <- function(object, ...) {
    UseMethod("getControl")
}

`getControl.default` <- function(object, ...) {
    nams <- names(object)
    if (!"control" %in% nams) {
        stop("Failed to find a 'control' component in 'object'.")
    }
    object[["control"]]
}

`getControl.allPerms` <- function(object, ...) {
    attr(object, "control")
}

`getHow` <- function(object, ...) {
    UseMethod("getControl")
}

library("testthat")
library("permute")

context("Testing get methods")

test_that("default methods for get functions", {
    v <- 1:10
    expect_error(getBlocks(v), regexp = "No default method")
    expect_error(getPlots(v), regexp = "No default method")
    expect_error(getWithin(v), regexp = "No default method")
    expect_error(getStrata(v), regexp = "No default method")
    expect_error(getType(v), regexp = "No default method")
    expect_error(getMirror(v), regexp = "No default method")
    expect_error(getConstant(v), regexp = "No default method")
    expect_error(getNperm(v), regexp = "No default method")
    expect_error(getMaxperm(v), regexp = "No default method")
    expect_error(getMinperm(v), regexp = "No default method")
    expect_error(getMake(v), regexp = "No default method")
    expect_error(getObserved(v), regexp = "No default method")
    expect_error(getAllperms(v), regexp = "No default method")
    expect_error(getComplete(v), regexp = "No default method")
    expect_error(getControl(v), regexp = "Failed to find a 'control' component in 'object'")
    expect_error(getHow(v), regexp = "Failed to find a 'control' component in 'object'")
})

test_that("getControl works for allperms", {
    ctrl <- how()
    ap <- allPerms(1:6, control = ctrl)
    CTRL <- getControl(ap)
    expect_is(CTRL, "how")
    CTRL <- getHow(ap)
    expect_is(CTRL, "how")
})

test_that("getHow works for check()", {
    ctrl <- how()
    res <- check(1:4, ctrl)
    CTRL <- getControl(res)
    expect_is(CTRL, "how")
    CTRL <- getHow(res)
    expect_is(CTRL, "how")
})

library("testthat")
library("permute")

context("Testing set<- methods")

test_that("default methods for get functions", {
    v <- 1:10
    expect_error(setNperm(v) <- 10, regexp = "No default method")
    expect_error(setMaxperm(v) <- 10, regexp = "No default method")
    expect_error(setMinperm(v) <- 10, regexp = "No default method")
    expect_error(setComplete(v) <- TRUE, regexp = "No default method")
    expect_error(setBlocks(v) <- TRUE, regexp = "No default method")
    expect_error(setAllperms(v) <- 1:10, regexp = "No default method")
    expect_error(setObserved(v) <- TRUE, regexp = "No default method")
    expect_error(setPlots(v) <- Plots(), regexp = "No default method")
    expect_error(setWithin(v) <- Within(), regexp = "No default method")
    expect_error(setStrata(v) <- gl(2, 5), regexp = "No default method")
    expect_error(setRow(v) <- 4, regexp = "No default method")
    expect_error(setCol(v) <- 5, regexp = "No default method")
    expect_error(setDim(v) <- c(2,3), regexp = "No default method")
    expect_error(setType(v) <- "series", regexp = "No default method")
    expect_error(setMirror(v) <- TRUE, regexp = "No default method")
    expect_error(setMake(v) <- TRUE, regexp = "No default method")
    expect_error(setConstant(v) <- "series", regexp = "No default method")
})

test_that("how() set methods throw errors where not appropriate for use", {
    h <- how()
    expect_error(setDim(h) <- c(2,3), regexp = "can not be used directly on '\"how\"' objects")
    expect_error(setMirror(h) <- TRUE, regexp = "can not be used directly on '\"how\"' objects")
    expect_error(setConstant(h) <- TRUE, regexp = "can not be used directly on '\"how\"' objects")
    expect_error(setType(h) <- "series", regexp = "can not be used directly on '\"how\"' objects")
    expect_error(setRow(h) <- 2, regexp = "can not be used directly on '\"how\"' objects")
    expect_error(setCol(h) <- 3, regexp = "can not be used directly on '\"how\"' objects")
})

test_that("set within for class how works", {
    h <- how()
    setWithin(h) <- Within(type = "series")
    expect_identical(getType(h, which = "within"), "series")
})

test_that("set plots for class how works", {
    h <- how()
    setPlots(h) <- Plots(type = "series")
    expect_identical(getType(h, which = "plots"), "series")
})

test_that("test setMinperm work", {
    h <- how()
    setMinperm(h) <- 999
    expect_is(h, "how")
    expect_equal(getMinperm(h), 999)
})

test_that("test setMaxperm work", {
    h <- how()
    nperm <- 99999
    setMaxperm(h) <- nperm
    expect_is(h, "how")
    expect_equal(getMaxperm(h), nperm)
})

test_that("test setStrata works", {
    h <- how()
    f <- gl(5,5)
    setStrata(h) <- f
    expect_is(h, "how")
    expect_identical(getStrata(h), f)

    plots <- getPlots(h)
    f <- gl(4,5)
    setStrata(plots) <- f
    expect_identical(getStrata(plots), f)
    setPlots(h) <- plots
    expect_identical(getStrata(h), f)
})

test_that("test setRow<- works", {
    f <- gl(9, 25)
    h <- how(within = Within(type = "grid", nrow = 5, ncol = 5),
             plots  =  Plots(type = "grid", nrow = 3, ncol = 3,
                             strata = f))
    w <- getWithin(h)
    setRow(w) <- 2
    expect_identical(getRow(w), 2L)
    expect_is(w, "Within")

    p <- getPlots(h)
    setRow(p) <- 4
    expect_identical(getRow(p), 4L)
    expect_is(p, "Plots")
})

test_that("test setCol<- works", {
    f <- gl(9, 25)
    h <- how(within = Within(type = "grid", nrow = 5, ncol = 5),
             plots  =  Plots(type = "grid", nrow = 3, ncol = 3,
                             strata = f))
    w <- getWithin(h)
    setCol(w) <- 2
    expect_identical(getCol(w), 2L)
    expect_is(w, "Within")

    p <- getPlots(h)
    setCol(p) <- 4
    expect_identical(getCol(p), 4L)
    expect_is(p, "Plots")
})

test_that("test setDim<- works", {
    f <- gl(9, 25)
    h <- how(within = Within(type = "grid", nrow = 5, ncol = 5),
             plots  =  Plots(type = "grid", nrow = 3, ncol = 3,
                             strata = f))
    w <- getWithin(h)
    setDim(w) <- c(2, 4)
    expect_identical(getDim(w), c(2L, 4L))
    expect_is(w, "Within")

    p <- getPlots(h)
    setDim(p) <- c(4, 3)
    expect_identical(getDim(p), c(4L, 3L))
    expect_is(p, "Plots")
})

test_that("test setType<- works", {
    f <- gl(9, 25)
    h <- how(within = Within(type = "grid", nrow = 5, ncol = 5),
             plots  =  Plots(type = "series", strata = f))
    w <- getWithin(h)
    setType(w) <- "free"
    expect_is(w, "Within")
    expect_identical(getType(w), "free")

    p <- getPlots(h)
    expect_error(setType(p) <- "strata",
                 regexp = "Invalid permutation type")
    setType(p) <- "none"
    expect_is(p, "Plots")
    expect_identical(getType(p), "none")
})

test_that("test setMirror<- works", {
    f <- gl(9, 25)
    h <- how(within = Within(type = "grid", nrow = 5, ncol = 5),
             plots  =  Plots(type = "series", strata = f))
    w <- getWithin(h)
    setMirror(w) <- TRUE
    expect_is(w, "Within")
    expect_true(getMirror(w))

    p <- getPlots(h)
    setMirror(p) <- TRUE
    expect_is(p, "Plots")
    expect_true(getMirror(p))
})

test_that("test setConstant<- works", {
    f <- gl(9, 25)
    h <- how(within = Within(type = "grid", nrow = 5, ncol = 5),
             plots  =  Plots(type = "series", strata = f))
    w <- getWithin(h)
    setConstant(w) <- TRUE
    expect_is(w, "Within")
    expect_true(getConstant(w))

    p <- getPlots(h)
    expect_error(setConstant(p) <- TRUE,
                 regexp = "setConstant` does not apply to '\"Plots\"' objects.")
})

## Non-exported function that can be used to fix up elements of a call
`fixupCall` <- function(object, element, value) {
    ## element must be character! Try to coerce
    element <- as.character(element)     ## Fails if not coercible
    .call <- getElement(object, "call")  ## get call from object
    .ll <- as.list(.call)                ## convert to a list to manipulate
    names(.ll) <- names(.call)           ## set names on the list
    ## if element not in names need to create it
    if (! element %in% names(.ll)) {
        new <- list(value)
        names(new) <- element
        .ll <- c(.ll, new)
    } else {
        ## otherwise just update it
        .ll[[element]] <- value              ## set element to value
    }
    .call <- as.call(.ll)                ## convert back to a call
    object$call <- .call                 ## push back into object
    object
}

## new version of shuffle() that allows for blocking
`shuffle` <- function(n, control = how()) {
    ## handle a vector, matrix, or data frame input; derive n from it
    if (((is.numeric(n) || is.integer(n) || is.factor(n) || is.character(n)) &&
         length(n) > 1L) ||
        is.matrix(n) ||
        is.data.frame(n)) {
        n <- nobs(n)
    }
    sn <- seq_len(n) ## sequence of samples in order of input

    ## get blocking, if any
    Block <- getStrata(control, which = "blocks")
    ## If no blocking, put all samples in same block
    if(is.null(Block)) {
        Block <- factor(rep(1, n))
    } else {
        ## There was blocking so update control to remove it
        ## as we don't need it in control at the within-block
        ## permutations performed in the loop
        control <- update(control, blocks = NULL)
    }

    ## split sn on basis of Block
    spln <- split(sn, Block)
    nb <- length(spln) ## number of blocks

    ## result list
    out <- vector(mode = "list", length = nb)

    ## loop over spln and shuffle in each split
    for(i in seq_len(nb)) {
        out[[i]] <- doShuffle(spln[[i]], control)
    }
    out <- unsplit(out, Block) ## undo the original splitting
    out
}

`doShuffle` <- function(ind, control) {
    ## collect strata at Plot level
    Pstrata <- getStrata(control, which = "plots", drop = TRUE)
    plotCTRL <- getPlots(control)
    ## ...which need to be reduced to only those for `ind`
    Pstrata <- Pstrata[ind]

    n <- length(ind)
    sn <- seq_len(n)

    ## if no strata at Plot level permute all samples using stated scheme
    if(is.null(Pstrata)) {
        perm <- shuffleNoStrata(n, control)
    } else {
        typeP <- getType(control, which = "plots")
        typeW <- getType(control, which = "within")

        ## permute Plot strata?
        if(isTRUE(all.equal(typeP, "none"))) { ## NO
            perm <- sn
        } else {                               ## YES
            flip <- runif(1L) < 0.5 ## logical, passed on & used only if mirroring
            perm <- shuffleStrata(Pstrata,
                                  type = typeP,
                                  mirror = plotCTRL$mirror,
                                  flip = flip,
                                  nrow = plotCTRL$nrow,
                                  ncol = plotCTRL$ncol)
        }

        ## permute the samples within Plot strata
        if(!isTRUE(all.equal(typeW, "none"))) { ## NOTE the `!`
            ## house keeping to track permuted strata - used later
            tab <- table(Pstrata[perm])
            levs <- names(tab) ## levels of Plot strata in this split

            ## use same permutation within each level of strata?
            withinCTRL <- getWithin(control)
            CONSTANT <- withinCTRL$constant
            if(isTRUE(CONSTANT)) {
                if(isTRUE(all.equal(typeW, "free"))) {
                    N <- unique(tab)[1L]
                    same.rand <- shuffleFree(N, N)
                } else if(isTRUE(all.equal(typeW, "series"))) {
                    start <- shuffleFree(n / length(levs), 1L)
                    flip <- runif(1L) < 0.5
                } else if(isTRUE(all.equal(typeW, "grid"))) {
                    start.row <- shuffleFree(withinCTRL$nrow, 1L)
                    start.col <- shuffleFree(withinCTRL$ncol, 1L)
                    flip <- runif(2L) < 0.5
                }
            } else {
                start <- start.row <- start.col <- flip <- NULL
            }

            ## copy perm at this stage
            tmp <- perm

            ## for each level of strata in this split, shuffle
            for(lv in levs) {
                ## must re-order strata here on basis of out as they
                ## may have been permuted above
                MATCH <- Pstrata[perm] == lv
                gr <- perm[MATCH]
                if((n.gr <- length(gr)) > 1) {
                    tmp[MATCH] <-
                        switch(typeW,
                               "free" = if(isTRUE(CONSTANT)) {
                                   gr[same.rand]
                               } else {
                                   gr[shuffleFree(n.gr, n.gr)]
                               },
                               "series" =
                               gr[shuffleSeries(seq_len(n.gr),
                                                mirror = withinCTRL$mirror,
                                                start = start, flip = flip)],
                               "grid" =
                               gr[shuffleGrid(nrow = withinCTRL$nrow,
                                              ncol = withinCTRL$ncol,
                                              mirror = withinCTRL$mirror,
                                              start.row = start.row,
                                              start.col = start.col,
                                              flip = flip)]
                               )
                }
            }
            perm <- tmp
        }
    }
    ind[perm]
}

`print.allPerms` <- function(x, ...) {
    dims <- dim(x)
    control <- attr(x, "control")
    observed <- attr(x, "observed")
    attributes(x) <- NULL
    dim(x) <- dims
    print(x)
    return(invisible(x))
}

##' @title Replicate and cbind all block-level permutations
##' @param x a list whose compontents are the set of all permutations
##' at the block level
##' @return a matrix
##' @author Gavin L. Simpson
`cbindAllPerms` <- function(x) {
    nb <- length(x) ## number of blocks

    ## allPerms has first block varying slowest, but expand.grid has
    ## first factor varying fastest. Hence we reverse `x` here, and
    ## also reverse `out` back again later
    x <- rev(x)

    ## prepares nb seqence vectors 1:`obs in block` for expand.grid
    rowind <- do.call(expand.grid,
                      lapply(x, function(i) seq_len(nrow(i))))

    ## index elements of x using the row indices - gives a list to cbind
    ## next. sapply() over-simplifies to wrong dimensions so not used.
    ## Note: the lapply() result is reversed with `rev` to undo the reversing
    ## of `x` earlier; ensures everything is correct block order.
    out <- rev(lapply(seq_len(nb), function(i, m, ind) m[[i]][ind[, i] ,],
                      m = x, ind = rowind))
    do.call(cbind, out) ## return
}

`shuffleStrata` <- function(strata, type, mirror = FALSE, start = NULL,
                            flip = NULL, nrow, ncol, start.row = NULL,
                            start.col = NULL) {
    ## drop unused levels
    strata <- droplevels(strata)
    LEVS <- levels(strata)
    lev <- nlevels(strata)
    ngr <- length(strata) / lev
    SEQ <- seq_len(lev)
    sp <- split(out <- seq_along(strata), strata)
    perm <- if(type == "free") {
        shuffleFree(lev, lev)
    } else if (type == "series") {
        shuffleSeries(SEQ, mirror = mirror, start = start,
                      flip = flip)
    } else if (type == "grid") {
        shuffleGrid(nrow = nrow, ncol = ncol, mirror = mirror,
                    start.row = start.row, start.col = start.col,
                    flip = flip)
    } else {
        stop("Invalid permutation type.")
    }
    for(i in SEQ) {
        want <- which(strata == LEVS[i])
        out[want] <- sp[[perm[i]]]
    }
    out
}

`shuffleGrid` <- function(nrow, ncol, mirror = FALSE, start.row = NULL,
                          start.col = NULL, flip = NULL) {
    if(is.null(start.row))
        start.row <- shuffleFree(nrow, 1L)
    if(is.null(start.col))
        start.col <- shuffleFree(ncol, 1L)
    ir <- seq(start.row, length=nrow) %% nrow
    ic <- seq(start.col, length=ncol) %% ncol
    if(!is.null(flip) && mirror) {
        if(any(flip)) {
            if(flip[1L])
                ir <- rev(ir)
            if(flip[2L])
                ic <- rev(ic)
        }
    } else {
        if (mirror) {
            if (runif(1L) < 0.5)
                ir <- rev(ir)
            if (runif(1L) < 0.5)
                ic <- rev(ic)
        }
    }
    rep(ic, each=nrow) * nrow + rep(ir, len=nrow*ncol) + 1L
}

`shuffleSeries` <- function(x, mirror = FALSE, start = NULL,
                            flip = NULL) {
    n <- length(x)
    if(is.null(start))
        start <- shuffleFree(n, 1L)
    out <- seq(start, length = n) %% n + 1L
    if(!is.null(flip) && mirror) {
        if(flip)
            out <- rev(out)
    } else {
        if(mirror && runif(1L) < 0.5)
            out <- rev(out)
    }
    x[out]
}

`shuffleFree` <- function(x, size) {
    # need to ensure x is length 1 when passed to sample.int
    # documented behaviour is to allow x = 10:20 say
    # so we need to allow this now even though I never use it like this in
    # the package code, except for a single example
    if (length(x) > 1L) {
        p <- sample.int(length(x), size, replace = FALSE)
        out <- x[p]
    } else {
        out <- sample.int(x, size, replace = FALSE)
    }
    out
}

## wrapper function when shuffling without any strata at all at any level
`shuffleNoStrata` <- function(n, control) {
    type <- control$within$type
    switch(type,
           "free" = shuffleFree(n, n),
           "series" = shuffleSeries(seq_len(n), mirror = control$within$mirror),
           "grid" = shuffleGrid(nrow = control$within$nrow,
           ncol = control$within$ncol, mirror = control$within$mirror),
           "none" = seq_len(n)
           )
}

library("testthat")
library("permute")

context("Testing shuffle-util functions")

test_that("shuffleFree works with a vector", {
    x <- 10:20
    p <- shuffleFree(x, 1)
    expect_identical(length(p), 1L)
    expect_true(p %in% x)
})

test_that("shuffelStrata works for various 'type's", {
    f <- gl(4,5)
    perm <- shuffleStrata(f, type = "free")
    expect_equal(length(perm), length(f))
    expect_true(all(perm <= 20))
    expect_true(all(perm >= 1))
    perm <- shuffleStrata(f, type = "series")
    expect_equal(length(perm), length(f))
    expect_true(all(perm <= 20))
    expect_true(all(perm >= 1))
    perm <- shuffleStrata(f, type = "grid", ncol = 2, nrow = 2)
    expect_equal(length(perm), length(f))
    expect_true(all(perm <= 20))
    expect_true(all(perm >= 1))
    expect_error(shuffleStrata(f, type = "unknown"),
                 regexp = "Invalid permutation type")
})

test_that("shuffleGrid works with only ncol and nrow args", {
    nr <- 5
    nc <- 5
    perm <- shuffleGrid(ncol = nc, nrow = nr)
    expect_equal(length(perm), prod(nr, nc))
})

test_that("shuffleGrid works with mirror & flip combinations", {
    nr <- 5
    nc <- 5
    perm <- shuffleGrid(ncol = nc, nrow = nr, mirror = TRUE)
    expect_equal(length(perm), prod(nr, nc))
    perm <- shuffleGrid(ncol = nc, nrow = nr, mirror = TRUE)
    expect_equal(length(perm), prod(nr, nc))
    perm <- shuffleGrid(ncol = nc, nrow = nr, mirror = TRUE,
                        flip = c(TRUE, FALSE))
    expect_equal(length(perm), prod(nr, nc))
    perm <- shuffleGrid(ncol = nc, nrow = nr, mirror = TRUE,
                        flip = c(FALSE, TRUE))
    expect_equal(length(perm), prod(nr, nc))
})

test_that("shuffleGrid works with start.row & start.col", {
    nr <- 5
    nc <- 5
    perm <- shuffleGrid(ncol = nc, nrow = nr,
                        start.row = 2)
    expect_equal(length(perm), prod(nr, nc))
    expect_true(all(perm <= 25))
    expect_true(all(perm >=1))
    perm <- shuffleGrid(ncol = nc, nrow = nr,
                        start.col = 4)
    expect_equal(length(perm), prod(nr, nc))
    expect_true(all(perm <= 25))
    expect_true(all(perm >=1))
    perm <- shuffleGrid(ncol = nc, nrow = nr,
                        start.col = 4, start.row = 2)
    expect_equal(length(perm), prod(nr, nc))
    expect_true(all(perm <= 25))
    expect_true(all(perm >=1))
})

### Extractor, replacement functions for blocks

### Extractor methods
`blocks` <- function(object, ...) {
    UseMethod("blocks")
}

`blocks.default` <- function(object, ...) {
    stop("No default method for 'blocks()'")
}

`blocks.how` <- function(object, ...) {
    object$blocks
}

### Replacement methods
`blocks<-` <- function(object, value) {
    UseMethod("setBlocks<-")
}

`blocks<-.default` <- function(object, value) {
    stop("No default method for `setBlocks`")
}

`blocks<-.how` <- function(object, value) {
    object[["blocks.name"]] <- deparse(substitute(value))
    if (!is.null(value))
        value <- as.factor(value)
    object["blocks"] <- list(value)
    object <- fixupCall(object, "blocks", value)
    object
}

### Temporarily drop this here
`Blocks` <- function(strata = NULL) {
    out <- list(strata = strata)
    ## keep as list for now
    ##class(out) <- "Blocks"
    out
}

library("testthat")
library("permute")

context("Testing blocks() and Blocks()")

test_that("Blocks() returns a list with 'strata'", {
    f <- gl(4, 10)
    b <- Blocks(f)
    expect_is(b, "list")
    expect_true("strata" %in% names(b))
})

## Replacement functions for blocks, plots and within, plus strata,
## etc ...
`setNperm<-` <- function(object, value) {
    UseMethod("setNperm<-")
}

`setNperm<-.default` <- function(object, value) {
    stop("No default method for `setNperm`")
}

`setNperm<-.how` <- function(object, value) {
    object[["nperm"]] <- value
    object <- fixupCall(object, "nperm", value)
    object
}

`setMaxperm<-` <- function(object, value) {
    UseMethod("setMaxperm<-")
}

`setMaxperm<-.default` <- function(object, value) {
    stop("No default method for `setMaxperm`")
}

`setMaxperm<-.how` <- function(object, value) {
    object[["maxperm"]] <- value
    object <- fixupCall(object, "maxperm", value)
    object
}

`setMinperm<-` <- function(object, value) {
    UseMethod("setMinperm<-")
}

`setMinperm<-.default` <- function(object, value) {
    stop("No default method for `setMinperm`")
}

`setMinperm<-.how` <- function(object, value) {
    object[["minperm"]] <- value
    object <- fixupCall(object, "minperm", value)
    object
}

`setComplete<-` <- function(object, value) {
    UseMethod("setComplete<-")
}

`setComplete<-.default` <- function(object, value) {
    stop("No default method for `setComplete`")
}

`setComplete<-.how` <- function(object, value) {
    if (!is.null(value))
        value <- rep(as.logical(value), length.out = 1)
    object[["complete"]] <- value
    object <- fixupCall(object, "complete", value)
    object
}

`setAllperms<-` <- function(object, value) {
    UseMethod("setAllperms<-")
}

`setAllperms<-.default` <- function(object, value) {
    stop("No default method for `setAllperms`")
}

`setAllperms<-.how` <- function(object, value) {
    if (!is.null(value)) {
        value <- as.allPerms(value, control = object)
    }
    object[["all.perms"]] <- value
    object <- fixupCall(object, "all.perms", value)
    object
}

`setMake<-` <- function(object, value) {
    UseMethod("setMake<-")
}

`setMake<-.default` <- function(object, value) {
    stop("No default method for `setMake`")
}

`setMake<-.how` <- function(object, value) {
    if (!is.null(value))
        value <- rep(as.logical(value), length.out = 1)
    object[["make"]] <- value
    object <- fixupCall(object, "make", value)
    object
}

`setBlocks<-` <- function(object, value) {
    UseMethod("setBlocks<-")
}

`setBlocks<-.default` <- function(object, value) {
    stop("No default method for `setBlocks`")
}

`setBlocks<-.how` <- function(object, value) {
    object[["blocks.name"]] <- deparse(substitute(value))
    if (!is.null(value))
        value <- as.factor(value)
    object["blocks"] <- list(value)
    object <- fixupCall(object, "blocks", value)
    object
}

`setObserved<-` <- function(object, value) {
    UseMethod("setObserved<-")
}

`setObserved<-.default` <- function(object, value) {
    stop("No default method for `setObserved`")
}

`setObserved<-.how` <- function(object, value) {
    if (!is.null(value))
        value <- rep(as.logical(value), length.out = 1)
    object[["observed"]] <- value
    object <- fixupCall(object, "observed", value)
    object
}

## Plots ##############################################################
`setPlots<-` <- function(object, value) {
    UseMethod("setPlots<-")
}

`setPlots<-.default` <- function(object, value) {
    stop("No default method for `setPlots`")
}

`setPlots<-.how` <- function(object, value) {
    stopifnot(inherits(value, "Plots"))
    object[["plots"]] <- value
    object <- fixupCall(object, "plots", getCall(value))
    object
}

## Within ##############################################################
`setWithin<-` <- function(object, value) {
    UseMethod("setWithin<-")
}

`setWithin<-.default` <- function(object, value) {
    stop("No default method for `setWithin`")
}

`setWithin<-.how` <- function(object, value) {
    stopifnot(inherits(value, "Within"))
    object[["within"]] <- value
    object <- fixupCall(object, "within", getCall(value))
    object
}

## Strata #############################################################
`setStrata<-` <- function(object, value) {
    UseMethod("setStrata<-")
}

`setStrata<-.default` <- function(object, value) {
    stop("No default method for `setStrata`")
}

`setStrata<-.how` <- function(object, value) {
    if (!is.null(value)) {
        value <- as.factor(value)
    }
    ## get Plots
    plots <- getPlots(object)
    setStrata(plots) <- value
    setPlots(object) <- plots
    object
}

`setStrata<-.Plots` <- function(object, value) {
    if (!is.null(value))
        value <- as.factor(value)
    object[["strata"]] <- value
    object <- fixupCall(object, "strata", value) # value was getCall(value))
    object
}

## Grid dimensions ####################################################
`setRow<-` <- function(object, value) {
    UseMethod("setRow<-")
}

`setRow<-.default` <- function(object, value) {
    stop("No default method for `setRow`")
}

`setRow<-.how` <- function(object, value) {
    stop("`setRow` can not be used directly on '\"how\"' objects.")
}

`setRow<-.Within` <- function(object, value) {
    value <- as.integer(value)
    object[["nrow"]] <- value
    object <- fixupCall(object, "nrow", value)
    object
}

`setRow<-.Plots` <- function(object, value) {
    value <- as.integer(value)
    object[["nrow"]] <- value
    object <- fixupCall(object, "nrow", value)
    object
}

`setCol<-` <- function(object, value) {
    UseMethod("setCol<-")
}

`setCol<-.default` <- function(object, value) {
    stop("No default method for `setCol`")
}

`setCol<-.how` <- function(object, value) {
    stop("`setCol` can not be used directly on '\"how\"' objects.")
}

`setCol<-.Within` <- function(object, value) {
    value <- as.integer(value)
    object[["ncol"]] <- value
    object <- fixupCall(object, "ncol", value)
    object
}

`setCol<-.Plots` <- function(object, value) {
    value <- as.integer(value)
    object[["ncol"]] <- value
    object <- fixupCall(object, "ncol", value)
    object
}

`setDim<-` <- function(object, value) {
    UseMethod("setDim<-")
}

`setDim<-.default` <- function(object, value) {
    stop("No default method for `setDim`")
}

`setDim<-.how` <- function(object, value) {
    stop("`setDim` can not be used directly on '\"how\"' objects.")
}

`setDim<-.Within` <- function(object, value) {
    value <- as.integer(value)
    stopifnot(all.equal(length(value), 2L))
    setRow(object) <- value[1]
    setCol(object) <- value[2]
    object
}

`setDim<-.Plots` <- function(object, value) {
    value <- as.integer(value)
    stopifnot(all.equal(length(value), 2L))
    setRow(object) <- value[1]
    setCol(object) <- value[2]
    object
}

## setType ############################################################
`setType<-` <- function(object, value) {
    UseMethod("setType<-")
}

`setType<-.default` <- function(object, value) {
    stop("No default method for `setType`")
}

`setType<-.how` <- function(object, value) {
    stop("`setType` can not be used directly on '\"how\"' objects.")
}

`setType<-.Within` <- function(object, value) {
    value <- as.character(value)
    if (!value %in% c("free","series","grid","none"))
        stop("Invalid permutation type")
    value <- rep(value, length.out = 1L)
    object[["type"]] <- value
    object <- fixupCall(object, "type", value)
    object
}

`setType<-.Plots` <- function(object, value) {
    value <- as.character(value)
    if (!value %in% c("free","series","grid","none"))
        stop("Invalid permutation type")
    value <- rep(value, length.out = 1L)
    object[["type"]] <- value
    object <- fixupCall(object, "type", value)
    object
}

## setMirror ############################################################
`setMirror<-` <- function(object, value) {
    UseMethod("setMirror<-")
}

`setMirror<-.default` <- function(object, value) {
    stop("No default method for `setMirror`")
}

`setMirror<-.how` <- function(object, value) {
    stop("`setMirror` can not be used directly on '\"how\"' objects.")
}

`setMirror<-.Within` <- function(object, value) {
    if (!is.null(value))
        value <- rep(as.logical(value), length.out = 1)
    object[["mirror"]] <- value
    object <- fixupCall(object, "mirror", value)
    object
}

`setMirror<-.Plots` <- function(object, value) {
    if (!is.null(value))
        value <- rep(as.logical(value), length.out = 1)
    object[["mirror"]] <- value
    object <- fixupCall(object, "mirror", value)
    object
}

## setConstant ############################################################
`setConstant<-` <- function(object, value) {
    UseMethod("setConstant<-")
}

`setConstant<-.default` <- function(object, value) {
    stop("No default method for `setConstant`")
}

`setConstant<-.how` <- function(object, value) {
    stop("`setConstant` can not be used directly on '\"how\"' objects.")
}

`setConstant<-.Within` <- function(object, value) {
    if (!is.null(value))
        value <- rep(as.logical(value), length.out = 1)
    object[["constant"]] <- value
    object <- fixupCall(object, "constant", value)
    object
}

`setConstant<-.Plots` <- function(object, value) {
    stop("`setConstant` does not apply to '\"Plots\"' objects.")
}

`allSeries` <- function(n, nperms, mirror = FALSE)
{
    v <- seq_len(n)
    X <- matrix(nrow = nperms, ncol = n)
    for(i in v) {
        X[i,] <- seq(i, length = n)%%n + 1
    }
    ## if mirroring, rev the cols of X[v,]
    ## but only if nperms > 2
    if(mirror && (nperms > 2))
        X[(n+1):(2*n),] <- X[v, rev(v)]
    X
}

`how` <- function(within = Within(),
                  plots = Plots(),
                  blocks = NULL,
                  nperm = 199,
                  complete = FALSE,
                  maxperm = 9999,
                  minperm = 5040,
                  all.perms = NULL,
                  make = TRUE,
                  observed = FALSE) {

    blocks.name <- deparse(substitute(blocks))
    ## blocks should also be a factor - coerce
    if(!is.null(blocks))
        blocks <- as.factor(blocks)

    ## process the call to make it standalone
    .call <- match.call()
    if (length(.call) > 1L) {
        .ll <- as.list(.call[-1])
        args <- names(.call)[-1]
        ## evaluate arguments other than within and plots
        ## those handled in their respective functions
        for (i in args[!args %in% c("within","plots")]) {
            if(!is.null(.ll[[i]])) {
                .ll[[i]] <- eval(.ll[[i]], parent.frame())
            }
        }
    }

    out <- list(within = within, plots = plots, blocks = blocks,
                nperm = nperm, complete = complete,
                maxperm = maxperm, minperm = minperm,
                all.perms = all.perms, make = make,
                observed = observed,
                blocks.name = blocks.name)

    ## process within and plots separately
    if (length(.call) > 1L && "within" %in% args) {
        .ll[["within"]] <- getCall(within)
    }
    if (length(.call) > 1L && "plots" %in% args) {
        .ll[["plots"]] <- getCall(plots)
    }

    ## finsh off
    if (length(.call) > 1L) {
        .ll <- c(as.list(.call[[1]]), .ll)
        names(.ll) <- names(.call)
        .call <- as.call(.ll)
    }

    out$call <- .call

    class(out) <- "how"
    out
}

`allStrata` <- function(n, control)
{
    ## seq vector of observation indices
    v <- seq_len(n)
    ## number of groups
    strata <- getStrata(control, which = "plots")
    lev <- length(levels(strata))
    ## compute nperms on number of levels - for this need Within()
    ## and type == typeP
    type <- getType(control, which = "plots")
    newControl <- how(within = Within(type = type))
    nperms <- numPerms(lev, newControl)
    ## result object
    X <- matrix(nrow = nperms, ncol = length(strata))
    ## store the type
    type <- getType(control, which = "plots")
    mirror <- getMirror(control, which = "plots")
    perms <- if(type == "free") {
        allFree(lev)
    } else if(type == "series") {
        allSeries(lev, nperms = nperms, mirror = mirror)
    } else if(type == "grid") {
        nr <- getRow(control, which = "plots")
        nc <- getCol(control, which = "plots")
        constant <- getConstant(control)
        allGrid(lev, nperms = nperms, nr = nr, nc = nc,
                mirror = mirror, constant = constant)
    } else {
        ## if in here, must have both types == "none"
        ## this is here just in case - need to check if this
        ## is possible given calling function...
        return(v)
    }
    sp <- split(v, strata)
    ## build permutations by permuting the split indices (as list)
    ## then undo the original splitting. This respects original indices
    ## of the samples, even where strata ar not contiguous
    for(i in seq_len(nrow(perms))) {
        X[i, ] <- unsplit(sp[perms[i, ]], strata)
    }
    X
}

`print.summary.allPerms` <- function(x, ...) {
    dims <- dim(x)
    control <- attr(x, "control")
    observed <- attr(x, "observed")
    attributes(x) <- NULL
    dim(x) <- dims
    cat("\n")
    writeLines(strwrap("Complete enumeration of permutations\n",
        prefix = "\t"))
    print(control)
    cat("\nAll permutations:\n")
    writeLines(paste("Contains observed ordering?:", ifelse(observed, "Yes", "No"),
              "\n"))
    print(x)
    return(invisible(x))
}

## Simple print method for objects of class "permutationMatrix"
##  - at the moment, don't print the attributes

`print.permutationMatrix` <- function(x, ...) {
    ## indicators of plot and block strata
    pl <- bl <- FALSE

    ## grab the permutation design
    ctrl <- attr(x, "control") ## gives us the list generated by how()

    blocks <- getBlocks(ctrl)
    plots <- getPlots(ctrl)
    strata <- getStrata(ctrl)

    ## print out dimensions of permutation matrix
    msg <- paste("No. of Permutations: ", nrow(x), sep = "")
    writeLines(strwrap(msg))

    ## print info on the within level
    msg <- paste("No. of Samples:", ncol(x), "(")
    if (any(pl <- !is.null(strata), bl <- !is.null(blocks))) {
        msg <- paste(msg, "Nested in: ", sep = "")
        if (pl && !bl) {
            nmsg <- "plots; "
        } else if (bl && !pl) {
            nmsg <- "blocks; "
        } else {
            nmsg <- "plots & blocks; "
        }
        msg <- paste(msg, nmsg, sep = "")
    }
    wmsg <- switch(wt <- getType(ctrl, which = "within"),
                   none = "",
                   free = "Randomised",
                   series = "Sequence",
                   grid = paste("Spatial grid: ",
                   getRow(ctrl, which = "within"), "r, ",
                   getCol(ctrl, which = "within"), "c", sep = ""))
    msg <- paste(msg, wmsg, sep = "")
    ## add info on mirroring if series or grid
    if ((wt %in% c("series", "grid")) && getMirror(ctrl, which = "within")) {
        msg <- paste(msg, "; mirrored", sep = "")
    }
    ## add info on constant
    if (getConstant(ctrl, which = "within") && pl) {
        msg <- paste(msg, "; same permutation in each plot", sep = "")
    }
    writeLines(strwrap(paste(msg, ")", sep = "")))

    ## print info on blocking, but ONLY if set
    if (!is.null(blocks <- getBlocks(ctrl))) {
        ll <- length(levels(blocks))
        msg <- paste("Restricted by Blocks: ", ctrl$blocks.name,
                     " (", ll, " ", if (ll == 1L) "block" else "blocks",
                     ")", sep = "")
        writeLines(strwrap(msg))
    }

    ## print info on plots, but ONLY if set
    if (!is.null(strata <- getStrata(ctrl, which = "plots"))) {
        #pl <- TRUE
        plots <- getPlots(ctrl)
        pmsg <- switch(pt <- getType(ctrl, which = "plots"),
                        none = "",
                        free = "; Randomised",
                        series = "; Sequence",
                        grid = paste("; Spatial grid: ",
                        getRow(ctrl, which = "plots"), "r, ",
                        getCol(ctrl, which = "plots"), "c", sep = ""))
        ## add info on mirroring if series or grid
        if ((pt %in% c("series","grid")) && getMirror(ctrl, which = "plots")) {
            pmsg <- paste(pmsg, " - mirrored")
        }
        ll <- length(levels(strata))
        msg <- paste("Restricted by Plots: ", plots$plots.name,
                     " (", ll, " ", if (ll == 1L) "plot" else "plots",
                     pmsg, ")", sep = "")
        writeLines(strwrap(msg))
    }


    cat("\n")
    x <- as.matrix(x)
    rownames(x) <- paste0("p", seq_len(nrow(x)))
    colnames(x) <- seq_len(ncol(x))
    print(x, ...)
}

## as.matrix.permutationMatrix - an S3 method to convert to the S3
## matrix class. Essentially this just strips attributes and updates
## the class to only "matrix"

`as.matrix.permutationMatrix` <- function(x, ...) {
    attr(x, "seed") <- NULL
    attr(x, "control") <- NULL
    class(x) <- "matrix"
    x
}

#  File permute/R/update.Plots.R
#  Part of the R package, http://www.R-project.org
#
#  Copyright (C) 1995-2012 The R Core Team
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.r-project.org/Licenses/
#
# Modifications by Gavin L. Simpson
#
# Copyright (C) 2013 Gavin L. Simpson
#
# Modifcations made:
#  1) Remove `formula.` argument and processing thereof.
#  2) Evaluation is forced to the global environment.
#  3) (minor) Took the definition of `call` out of the `if ()` statement
#     for clarity/style issues.
#  4) Added this modification section to the copyright/licence header.
#  5) Added code to preserve some components of the original object.

`update.Plots` <- function (object, ..., evaluate = TRUE) {
    call <- getCall(object)
    if (is.null(call))
        stop("need an object with call component")
    extras <- match.call(expand.dots = FALSE)$...

    ## preserve or update the plots names
    pname <- if ("strata" %in% names(extras)) {
        deparse(substitute(extras[["strata"]]))
    } else {
        object$plots.name
    }

    if (length(extras)) {
        existing <- !is.na(match(names(extras), names(call)))
        ## do these individually to allow NULL to remove entries.
        for (a in names(extras)[existing])
            call[[a]] <- extras[[a]]
        if (any(!existing)) {
            call <- c(as.list(call), extras[!existing])
            call <- as.call(call)
        }
    }
    if (evaluate) {
        out <- eval(call, parent.frame())
        out$plots.name <- pname
    } else {
        out <- call
    }

    out
}

# TODO List

A TODO list for **permute** - or things I know are broken or needed.

 * `summary.allPerms` - is not printing the permutation scheme.
   *Done in 0.7-5*

 * `print.permControl` - this needs to be updated to new API, and I don't
   like the `*** Foo ***` headings I used before. *Done in 0.7-3*

 * Need a way to update the permutation scheme, e.g. when a control
   object already exists and want to tweak it. Example is in `?allPerms`
   where I turn mirroring on via

        ctrl$within$mirror <- TRUE

    But how? Best idea currently is an `update.permControl` method. The
    generic is in namespace `stats`. Or a `modify()` function, perhaps
    coupled with specific replacement functions for certain components.
   
    *DONE, in part, in 0.7-5* The matched call is now returned by `how()`
    and this allows `update()` to do its work with no further effort from
    me. What isn't so neat is that currently this means you need to type
    out in full any specification of `within` and `plots` as these take
    the results of function calls. Hence we have, from `./man/how.Rd`
    
        plts <- gl(4,10)
        blks <- gl(2,20)
        h1 <- how(within = Within(type = "series", mirror = TRUE),
                  plots = Plots(strata = plts, type = "series"),
                  blocks = blks)

        ## The design can be updated...
        ## ... remove the blocking:
        update(h1, blocks = NULL)
        ## ... or switch the type of shuffling at a level:
        update(h1, plots = Plots(strata = plts, type = "none"))
        
    Where in the second `update()` the entire `Plots()` call needs to
    repeated to change just one part, the `type`.

    This has been tweaked a bit. The second example can now be done via:
    
        update(h1, plots = update(getPlots(h1), type = "none"))
    
    Here `getPlots(h1)` returns the `plots` component, which too has 
    a `call` component and hence can be `update()`ed, hence the nested 
    calls to `update()`.
         
 * `permControl` - deprecate this in favour of `how` as in "how to 
   permute"? *DONE Completed in 0.7-4*

 * `permuplot` - this may be fundamentally bust - it only worked in the
   original API and never got updated. Now the API is 2 versions further
   on! It is also more complex than the original - not sure it'll be
   easy without a lot of work to visualise all possible schemes.

 * `check` insists on returning all permutations *without* the observed
   one.

   *DONE Completed somewhere, probably when I made observed only work via
   how()*

 * The following example from `shuffleSet()` gives same number of 
   permutations via `numPerms()` for both designs; should be fewer if 
   same permutation within each plot.
   
        ## 10 random permutations in presence of Plot-level strata
        plotStrata <- Plots(strata = gl(4,5))
        CTRL <- how(plots = plotStrata,
                    within = Within(type = "free"))
        shuffleSet(20, 10, control = CTRL)
        ## as above but same random permutation within Plot-level strata
        CTRL <- how(plots = plotStrata,
                    within = Within(type = "free", constant = TRUE))
        shuffleSet(20, 10, CTRL)

    *DONE in 0.8-0*

 * Write an Rd page for the `"permutationMatrix"` S3 class where I can 
   describe the object returned by `shuffleSet()` and the methods 
   available for it.

    *DONE in 0.7-8*

---
title: "Restricted permutations; using the permute package"
author: "Gavin L. Simpson"
date: "`r Sys.Date()`"
output: bookdown::html_vignette2
bibliography: permute.bib  
vignette: >
  %\VignetteIndexEntry{Restricted permutations; using the permute package}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r, setup, echo = FALSE, results = "hide", message = FALSE, cache = FALSE}
knitr::opts_chunk$set(message = FALSE, warning = FALSE)
library("permute")
```

# Introduction

In classical frequentist statistics, the significance of a relationship or model is determined by reference to a null distribution for the test statistic. This distribution is derived mathematically and the probability of achieving a test statistic as large or larger than the observed statistic *if* the null hypothesis were true is looked-up from this null distribution. In deriving this probability, some assumptions about the data or the errors are made. If these assumptions are violated, then the validity of the derived $p$-value may be questioned.

An alternative to deriving the null distribution from theory is to generate a null distribution of the test statistic by randomly shuffling the data in some manner, refitting the model and deriving values for the test statistic for the permuted data. The level of significance of the test can be computed as the proportion of values of the test statistic from the null distribution that are equal to or larger than the observed value.

In many data sets, simply shuffling the data at random is inappropriate; under the null hypothesis, the data may not be freely exchangeable, for example if there is temporal or spatial correlation, or the samples are clustered in some way, such as multiple samples collected from each of a number of fields. The *permute* package was designed to provide facilities for generating these restricted permutations for use in randomisation tests. *permute* takes as its motivation the permutation schemes originally available in *Canoco* version 3.1 [@canoco31], which employed the cyclic- or toroidal-shifts suggested by @besagclifford.

# Simple randomisation {#sec:simple}
As an illustration of both randomisation and simple usage of the *permute* package we consider a small data set of mandible length measurements on specimens of the golden jackal (*Canis aureus*) from the British Museum of Natural History, London, UK. These data were collected as part of a study comparing prehistoric and modern canids [@higham80], and were analysed by @manly07. There are ten measurements of mandible length on both male and female specimens. The data are available in the `jackal` data frame supplied with *permute*.

```{r, load_jackal}
library("permute")
data(jackal)
jackal
```

The interest is whether there is a difference in the mean mandible length between male and female golden jackals. The null hypothesis is that there is zero difference in mandible length between the two sexes or that females have larger mandibles. The alternative hypothesis is that males have larger mandibles. The usual statistical test of this hypothesis is a one-sided $t$ test, which can be applied using `t.test()`

```{r, ttest_jackal}
jack.t <- t.test(Length ~ Sex, data = jackal, var.equal = TRUE,
                 alternative = "greater")
jack.t
```

The observed $t$ is `r with(jack.t, round(statistic, 3))` with `r with(jack.t, parameter)` df. The probability of observing a value this large or larger if the null hypothesis were true is `r with(jack.t, round(p.value, 4))`. Several assumptions have been made in deriving this $p$-value, namely

* random sampling of individuals from the populations of interest,
* equal population standard deviations for males and females, and
* that the mandible lengths are normally distributed within the sexes.

Assumption 1 is unlikely to be valid for museum specimens such as these, that have been collected in some unknown manner. Assumption 2 may be valid, Fisher's $F$-test and a Fligner-Killeen test both suggest that the standard deviations of the two populations do not differ significantly
```{rftest_jackal}
var.test(Length ~ Sex, data = jackal)
fligner.test(Length ~ Sex, data = jackal)
```
This assumption may be relaxed using `var.equal = FALSE` (the default) in the call to `t.test()`, to employ Welch's modification for unequal variances. Assumption 3 may be valid, but with such a small sample we are unable to reliably test this.

A randomisation test of the same hypothesis can be performed by randomly allocating ten of the mandible lengths to the male group and the remaining lengths to the female group. This randomisation is justified under the null hypothesis because the observed difference in mean mandible length between the two sexes is just a typical value for the difference in a sample if there were no difference in the population. An appropriate test statistic needs to be selected. We could use the $t$ statistic as derived in the $t$-test. Alternatively, we could base our randomisation test on the difference of means $D_i$ (male - female).

The main function in *permute* for providing random permutations is `shuffle()`. We can write our own randomisation test for the `jackal` data by first creating a function to compute the difference of means for two groups
```{r, meanFun}
meanDif <- function(x, grp) {
  mean(x[grp == "Male"]) - mean(x[grp == "Female"])
}
```
which can be used in a simple `for()` loop to generate the null distribution for the difference of means. First, we allocate some storage to hold the null difference of means; here we use 4999 random permutations so allocate a vector of length 5000. Then we iterate, randomly generating an ordering of the `Sex` vector and computing the difference of means for that permutation.
```{r, randJackal}
Djackal <- numeric(length = 5000)
N <- nrow(jackal)
set.seed(42)
for(i in seq_len(length(Djackal) - 1)) {
    perm <- shuffle(N)
    Djackal[i] <- with(jackal, meanDif(Length, Sex[perm]))
}
Djackal[5000] <- with(jackal, meanDif(Length, Sex))
```
The observed difference of means was added to the null distribution, because under the null hypothesis the observed allocation of mandible lengths to male and female jackals is just one of the possible random allocations.

The null distribution of $D_i$ can be visualised using a histogram, as shown in Figure~\@ref{draw_hist_jackal}. The observed difference of means (`r round(Djackal[5000], 2)`) is indicated by the red tick mark.

```{r, hist_jackal, fig=FALSE, echo=TRUE, eval=FALSE}
hist(Djackal, main = "",
     xlab = expression("Mean difference (Male - Female) in mm"))
rug(Djackal[5000], col = "red", lwd = 2)
```
The number of values in the randomisation distribution equal to or larger than the observed difference is
```{r}
(Dbig <- sum(Djackal >= Djackal[5000]))
```
giving a permutational $p$-value of
```{r}
Dbig / length(Djackal)
```
which is comparable with that determined from the frequentist $t$-test, and indicates strong evidence against the null hypothesis of no difference.

```{r, draw_hist_jackal, fig=TRUE, echo=FALSE, fig.cap="Distribution of the difference of mean mandible length in random allocations, ten to each sex."}
<<hist_jackal>>
```

In total there $^{20}C_{10} = `r formatC(choose(20,10), big.mark = ",", format = "f", digits = 0)`$ possible allocations of the 20 observations to two groups of ten
```{r, }
choose(20, 10)
```
so we have only evaluated a small proportion of these in the randomisation test.

The main workhorse function we used above was `shuffle()`. In this example, we could have used the base R function `sample()` to generate the randomised indices `perm` that were used to permute the `Sex` factor. Where `shuffle()` comes into it's own is for generating permutation indices from restricted permutation designs.

# The `shuffle()` and `shuffleSet()` functions
In the previous section I introduced the `shuffle()` function to generate permutation indices for use in a randomisation test. Now we will take a closer look at `shuffle()` and explore the various restricted permutation designs from which it can generate permutation indices.

`shuffle()` has two arguments:

i. `n`, the number of observations in the data set to be permuted, and
ii. `control`, a list that defines the permutation design describing how the samples should be permuted.

```{r, show_args}
args(shuffle)
```
A series of convenience functions are provided that allow the user to set-up even quite complex permutation designs with little effort. The user only needs to specify the aspects of the design they require and the convenience functions ensure all configuration choices are set and passed on to `shuffle()`. The main convenience function is `how()`, which returns a list specifying all the options available for controlling the sorts of permutations returned by `shuffle()`.
```{r, show_str}
str(how())
```
The defaults describe a random permutation design where all objects are freely exchangeable. Using these defaults, `shuffle(10)` amounts to `sample(1:10, 10, replace = FALSE)`:
```{r, compare_shuffle_sample}
set.seed(2)
(r1 <- shuffle(10))
set.seed(2)
(r2 <- sample(1:10, 10, replace = FALSE))
all.equal(r1, r2)
```

## Generating restricted permutations
Several types of permutation are available in *permute*:

* Free permutation of objects
* Time series or line transect designs, where the temporal or spatial ordering is preserved.
* Spatial grid designs, where the spatial ordering is preserved in both coordinate directions
* Permutation of plots or groups of samples.
* Blocking factors which restrict permutations to within blocks. The preceding designs can be nested within blocks.

The first three of these can be nested within the levels of a factor or to the levels of that factor, or to both. Such flexibility allows the analysis of split-plot designs using permutation tests, especially when combined with blocks.

`how()` is used to set up the design from which `shuffle()` will draw a permutation. `how()` has two main arguments that specify how samples are permuted *within* plots of samples or at the plot level itself. These are `within` and `plots`. Two convenience functions, `Within()` and `Plots()` can be used to set the various options for permutation. Blocks operate at the uppermost level of this hierarchy; blocks define groups of plots, each of which may contain groups of samples.

For example, to permute the observations `1:10` assuming a time series design for the entire set of observations, the following control object would be used

```{r, series1}
set.seed(4)
x <- 1:10
CTRL <- how(within = Within(type = "series"))
perm <- shuffle(10, control = CTRL)
perm
x[perm] ## equivalent
```

It is assumed that the observations are in temporal or transect order. We only specified the type of permutation within plots, the remaining options were set to their defaults via `Within()`.

A more complex design, with three plots, and a 3 by 3 spatial grid arrangement within each plot can be created as follows

```{r, grid1}
set.seed(4)
plt <- gl(3, 9)
CTRL <- how(within = Within(type = "grid", ncol = 3, nrow = 3),
            plots = Plots(strata = plt))
perm <- shuffle(length(plt), control = CTRL)
perm
```

Visualising the permutation as the 3 matrices may help illustrate how the data have been shuffled

```{r, vis_grid1, keep.source=TRUE}
## Original
lapply(split(seq_along(plt), plt), matrix, ncol = 3)
## Shuffled
lapply(split(perm, plt), matrix, ncol = 3)
```

In the first grid, the lower-left corner of the grid was set to row 2 and column 2 of the original, to row 1 and column 2 in the second grid, and to row 3 column 2 in the third grid.

To have the same permutation within each level of `plt`, use the `constant` argument of the `Within()` function, setting it to `TRUE`
```{r, grid_2, keep.source=TRUE}
set.seed(4)
CTRL <- how(within = Within(type = "grid", ncol = 3, nrow = 3,
                            constant = TRUE),
            plots = Plots(strata = plt))
perm2 <- shuffle(length(plt), control = CTRL)
lapply(split(perm2, plt), matrix, ncol = 3)
```

## Generating sets of permutations with shuffleSet()
There are several reasons why one might wish to generate a set of $n$ permutations instead of repeatedly generating permutations one at a time. Interpreting the permutation design happens each time `shuffle()` is called. This is an unnecessary computational burden, especially if you want to perform tests with large numbers of permutations. Furthermore, having the set of permutations available allows for expedited use with other functions, they can be iterated over using `for` loops or the `apply` family of functions, and the set of permutations can be exported for use outside of R.

The `shuffleSet()` function allows the generation of sets of permutations from any of the designs available in *permute*. `shuffleSet()` takes an additional argument to that of `shuffle()`, `nset`, which is the number of permutations required for the set. `nset` can be missing, in which case the number of permutations in the set is looked for in the object passed to `control`; using this, the desired number of permutations can be set at the time the design is created via the `nperm` argument of `how()`. For example,

```{r, series_2, results="hide"}
how(nperm = 10, within = Within(type = "series"))
```

Internally, `shuffle()` and `shuffleSet()` are very similar, with the major difference being that `shuffleSet()` arranges repeated calls to the workhorse permutation-generating functions, only incurring the overhead associated with interpreting the permutation design once. `shuffleSet()` returns a matrix where the rows represent different permutations in the set.

As an illustration, consider again the simple time series example from earlier. Here I generate a set of 5 permutations from the design, with the results returned as a matrix

```{r, shuffleSet_1}
set.seed(4)
CTRL <- how(within = Within(type = "series"))
pset <- shuffleSet(10, nset = 5, control = CTRL)
pset
```

It is worth taking a moment to explain what has happened here, behind the scenes. There are only `r numPerms(10, CTRL)` unique orderings (including the observed) in the set of permutations for this design. Such a small set of permutations triggers[^1] the generation of the entire set of permutations. From this set, `shuffleSet()` samples at random `nset` permutations. Hence the same number of random values has been generated via the pseudo-random number generator in *R* but we ensure a set of unique permutations is drawn, rather than randomly sample from a small set.

[^1]: The trigger is via the utility function `check()`, which calls another utility function, `allPerms()`, to generate the set of permutations for the stated design. The trigger for complete enumeration is set via `how()` using argument `minperm`; below this value, by default `check()` will generate the entire set of permutations.

# Defining permutation designs
In this section I give examples how various permutation designs can be specified using `how()`. It is not the intention to provide exhaustive coverage of all possible designs that can be produced; such a list would be tedious to both write *and* read. Instead, the main features and options will be described through a series of examples. The reader should then be able to put together the various options to create the exact structure required.

## Set the number of permutations
It may be useful to specify the number of permutations required in a permutation test alongside the permutation design. This is done via the `nperm` argument, as seen earlier. If nothing else is specified
```{r, results="hide"}
how(nperm = 999)
```
would indicate 999 random permutations where the samples are all freely exchangeable.

One advantage of using `nperm` is that `shuffleSet()` will use this if the `nset` argument is not specified. Additionally, `shuffleSet()` will check to see if the desired number of permutations is possible given the data and the requested design. This is done via the function `check()`, which is discussed later.

## The levels of the permutation hierarchy
There are three levels at which permutations can be controlled in *permute*. The highest level of the hierarchy is the *block* level. Blocks are defined by a factor variable. Blocks restrict permutation of samples to within the levels of this factor; samples are never swapped between blocks.

The *plot* level sits below blocks. Plots are defined by a factor and group samples in the same way as blocks. As such, some permutation designs can be initiated using a factor at the plot level or the same factor at the block level. The major difference between blocks and plots is that plots can also be permuted, whereas blocks are never permuted.

The lowest level of a permutation design in the *permute* hierarchy is known as *within*, and refers to samples nested *within* plots. If there are no plots or blocks, how samples are permuted at the *within* level applies to the entire data set.

### Permuting samples at the lowest level

How samples at the *within* level are permuted is configured using the `Within()` function. It takes the following arguments
```{r, withinArgs, echo=FALSE}
args(Within)
```

`type`
 ~ controls how the samples at the lowest level are permuted. The default is to form unrestricted permutations via option `"type"`. Options `"series"` and `"grid"` form restricted permutations via cyclic or toroidal shifts, respectively. The former is useful for samples that are a time series or line-transect, whilst the latter is used for samples on a regular spatial grid. The final option, `"none"`, will result in the samples at the lowest level not being permuted at all. This option is only of practical use when there are plots within the permutation/experimental design[^As blocks are never permuted, using `type = "none"` at the *within* level is also of no practical use.].

`constant`
 ~ this argument only has an effect when there are plots in the design[^Owing to the current implementation, whilst this option could also be useful when blocks to define groups of samples, it will not have any influence over how permutations are generated. As such, only use blocks for simple blocking structures and use plots if you require greater control of the permutations at the group (i.e. plot) level.]. `constant = TRUE` stipulates that each plot should have the same *within-plot* permutation. This is useful for example when you have time series of observations from several plots. If all plots were sampled at the same time points, it can be argued that at the plot level, the samples experienced the same *time* and hence the same permutation should be used within each plot.

`mirror`
 ~ when `type` is `"series"` or `"grid"`, argument `"mirror"` controls whether permutations are taken from the mirror image of the observed ordering in space or time. Consider the sequence `1, 2, 3, 4`. The relationship between observations is also preserved if we reverse the original ordering to `4, 3, 2, 1` and generate permutations from both these orderings. This is what happens when `mirror = TRUE`. For time series, the reversed ordering `4, 3, 2, 1` would imply an influence of observation 4 on observation 3, which is implausible. For spatial grids or line transects, however, this is a sensible option, and can significantly increase the number of possible permutations[^Setting `mirror = TRUE` will double or quadruple the set of permutations for `"series"` or `"grid"` permutations, respectively, as long as there are more than two time points or columns in the grid.].

`ncol`, `nrow`
 ~ define the dimensions of the spatial grid.

How `Within()` is used has already been encountered in earlier sections of this vignette; the function is used to supply a value to the `within` argument of `how()`. You may have noticed that all the arguments of `Within()` have default values? This means that the user need only supply a modified value for the arguments they wish to change. Also, arguments that are not relevant for the type of permutation stated are simply ignored; `nrow` and `ncol`, for example, could be set to any value without affecting the permutation design if `type != "grid"`[^No warnings are currently given if incompatible arguments are specified; they are ignored, but may show up in the printed output. This infelicity will be removed prior to *permute* version 1.0-0 being released.].

### Permuting samples at the Plot level

Permutation of samples at the *plot* level is configured via the `Plots()` function. As with `Within()`, `Plots()` is supplied to the `plots` argument of `how()`. `Plots()` takes many of the same arguments as `Within()`, the two differences being `strata`, a factor variable that describes the grouping of samples at the *plot* level, and the absence of a `constant` argument. As the majority of arguments are similar between `Within()` and `Plots()`, I will not repeat the details again, and only describe the `strata` argument

`strata`
 ~ a factor variable. `strata` describes the grouping of samples at the *plot* level, where samples from the same *plot* are take the same *level* of the factor.

When a *plot*-level design is specified, samples are never permuted between *plots*, only within plots if they are permuted at all. Hence, the type of permutation *within* the *plots* is controlled by `Within()`. Note also that with `Plots()`, the way the individual *plots* are permuted can be from any one of the four basic permutation types; `"none"`, `"free"`, `"series"`, and `"grid"`, as described above. To permute the *plots* only (i.e. retain the ordering of the samples *within* plots), you also need to specify `Within(type = "none", ...)` as the default in `Within()` is `type = "free"`. The ability to permute the plots whilst preserving the within-plot ordering is an important feature in testing explanatory factors at the whole-plot level in split-plot designs and in multifactorial analysis of variance [@canoco5manual].

### Specifying blocks; the top of the *permute* hierarchy

In contrast to the *within* and *plots* levels, the *blocks* level is simple to specify; all that is required is an indicator variable the same length as the data. Usually this is a factor, but `how()` will take anything that can be coerced to a factor via `as.factor()`.

It is worth repeating what the role of the block-level structure is; blocks simply restrict permutation to *within*, and never between, blocks, and blocks are never permuted. This is reflected in the implementation; the *split*-*apply*-`combine` paradigm is used to split on the blocking factor, the plot- and within-level permutation design is applied separately to each block, and finally the sets of permutations for each block are recombined.

## Examples
To do.

# Using *permute* in *R* functions

*permute* originally started life as a set of functions contained within the *vegan* package [@vegan] designed to provide a replacement for the `permuted.index()` function. From these humble origins, I realised other users and package authors might want to make use of the code I was writing and so Jari Oksanen, the maintainer of *vegan*, and I decided to spin off the code into the *permute* package. Hence from the very beginning, *permute* was intended for use both by users, to defining permutation designs, and by package authors, with which to implement permutation tests within their packages.

In the previous sections, I described the various user-facing functions that are employed to set up permutation designs and generate permutations from these. Here I will outline how package authors can use functionality in the *permute* package to implement permutation tests.

In Section~\@ref{sec:simple} I showed how a permutation test function could be written using the `shuffle()` function and allowing the user to pass into the test function an object created with `how()`. As mentioned earlier, it is more efficient to generate a set of permutations via a call to `shuffleSet()` than to repeatedly call `shuffle()` and large number of times. Another advantage of using `shuffleSet()` is that once the set of permutations has been created, parallel processing can be used to break the set of permutations down into smaller chunks, each of which can be worked on simultaneously. As a result, package authors are encouraged to use `shuffleSet()` instead of the simpler `shuffle()`.

To illustrate how to use *permute* in *R* functions, I'll rework the permutation test I used for the `jackal` data earlier in Section~\@ref{sec:simple}.

```{r, ptest-fun}
pt.test <- function(x, group, nperm = 199) {
    ## mean difference function
    meanDif <- function(i, x, grp) {
        grp <- grp[i]
        mean(x[grp == "Male"]) - mean(x[grp == "Female"])
    }
    ## check x and group are of same length
    stopifnot(all.equal(length(x), length(group)))
    ## number of observations
    N <- nobs(x)
    ## generate the required set of permutations
    pset <- shuffleSet(N, nset = nperm)
    ## iterate over the set of permutations applying meanDif
    D <- apply(pset, 1, meanDif, x = x, grp = group)
    ## add on the observed mean difference
    D <- c(meanDif(seq_len(N), x, group), D)
    ## compute & return the p-value
    Ds <- sum(D >= D[1]) # how many >= to the observed diff?
    Ds / (nperm + 1)     # what proportion of perms is this (the pval)?
}
```

The commented function should be reasonably self explanatory. I've altered the in-line version of the `meanDif()` function to take a vector of permutation indices `i` as the first argument, and internally the `grp` vector is permuted according to `i`. The other major change is that `shuffleSet()` is used to generate a set of permutations, which are then iterated over using `apply()`.

In use we see
```{r, run-ptest}
set.seed(42) ## same seed as earlier
pval <- with(jackal, pt.test(Length, Sex, nperm = 4999))
pval
```
which nicely agrees with the test we did earlier by hand.

Iterating over a set of permutation indices also means that adding parallel processing of the permutations requires only trivial changes to the main function code. As an illustration, below I show a parallel version of `pt.test()`

```{r, parallel-ptest-fun}
ppt.test <- function(x, group, nperm = 199, cores = 2) {
    ## mean difference function
    meanDif <- function(i, .x, .grp) {
        .grp <- .grp[i]
        mean(.x[.grp == "Male"]) - mean(.x[.grp == "Female"])
    }
    ## check x and group are of same length
    stopifnot(all.equal(length(x), length(group)))
    ## number of observations
    N <- nobs(x)
    ## generate the required set of permutations
    pset <- shuffleSet(N, nset = nperm)
    if (cores > 1) {
        ## initiate a cluster
        cl <- makeCluster(cores)
        on.exit(stopCluster(cl = cl))
        ## iterate over the set of permutations applying meanDif
        D <- parRapply(cl, pset, meanDif, .x = x, .grp = group)
    } else {
        D <- apply(pset, 1, meanDif, .x = x, .grp = group)
    }
    ## add on the observed mean difference
    D <- c(meanDif(seq_len(N), x, group), D)
    ## compute & return the p-value
    Ds <- sum(D >= D[1]) # how many >= to the observed diff?
    Ds / (nperm + 1)     # what proportion of perms is this (the pval)?
}
```

In use we observe
```{r, run-pptest}
require("parallel")
set.seed(42)
system.time(ppval <- ppt.test(jackal$Length, jackal$Sex, nperm = 9999,
                              cores = 2))
ppval
```
In this case there is little to be gained by splitting the computations over two CPU cores
```{r, run-pptest2}
set.seed(42)
system.time(ppval2 <- ppt.test(jackal$Length, jackal$Sex, nperm = 9999,
                               cores = 1))
ppval2
```
The cost of setting up and managing the parallel processes, and recombining the separate sets of results almost negates the gain in running the permutations in parallel. Here, the computations involved in `meanDif()` are trivial and we would expect greater efficiencies from running the permutations in parallel for more complex analyses.

## Accessing and changing permutation designs

The object created by `how()` is a relatively simple list containing the settings for the specified permutation design. As such one could use the standard subsetting and replacement functions in base *R* to alter components of the list. This is not recommended, however, as the internal structure of the list returned by `how()` may change in a later version of *permute*. Furthermore, to facilitate the use of `update()` at the user-level to alter the permutation design in a user-friendly way, the matched `how()` call is stored within the list along with the matched calls for any `Within()` or `Plots()` components. These matched calls need to be updated too if the list describing the permutation design is altered. To allow function writers to access and alter permutation designs, *permute* provides a series of extractor and replacement functions that have the forms `getFoo()` and `setFoo<-()`, respectively,where `Foo` is replaced by a particular component to be extracted or replaced.

The `getFoo()` functions provided by *permute* are

`getWithin()`, `getPlots()`, `getBlocks()`
 ~ these extract the details of the *within*-, *plots*-, and *blocks*-level components of the design. Given the current design (as of *permute* version 0.8-0), the first two of these return lists with classes `"Within"` and `"Plots"`, respectively, whilst `getBlocks()` returns the block-level factor.

`getStrata()`
 ~ returns the factor describing the grouping of samples at the *plots* or *blocks* levels, as determined by the value of argument `which`.

`getType()`
 ~ returns the type of permutation of samples at the *within* or *plots* levels, as determined by the value of argument `which`.

`getMirror()`
 ~ returns a logical, indicating whether permutations are drawn from the mirror image of the observed ordering at the *within* or *plots* levels, as determined by the value of argument `which`.

`getConstant()`
 ~ returns a logical, indicating whether the same permutation of samples, or a different permutation, is used within each of the plots.

`getRow()`, `getCol()`, `getDim()`
 ~ return dimensions of the spatial grid of samples at the *plots* or *blocks* levels, as determined by the value of argument `which`.

`getNperm()`, `getMaxperm()`, `getMinperm()`
 ~ return numerics for the stored number of permutations requested plus two triggers used when checking permutation designs via `check()`.

`getComplete()`
 ~ returns a logical, indicating whether complete enumeration of the set of permutations was requested.

`getMake()`
 ~ returns a logical, indicating whether the entire set of permutations should be produced or not.

`getObserved()`
 ~ returns a logical, which indicates whether the observed permutation (ordering of samples) is included in the entire set of permutation generated by `allPerms()`.

`getAllperms()`
 ~ extracts the complete set of permutations if present. Returns `NULL` if the set has not been generated.

The available `setFoo()<-` functions are

`setPlots<-()`, `setWithin<-()`
 ~ replaces the details of the *within*-, and *plots*-, components of the design. The replacement object must be of class `"Plots"` or `"Within"`, respectively, and hence is most usefully used in combination with the `Plots()` or `Within()` constructor functions.

`setBlocks<-()`
 ~ replaces the factor that partitions the observations into blocks. `value` can be any *R* object that can be coerced to a factor vector via `as.factor()`.

`setStrata<-()`
 ~ replaces either the `blocks` or `strata` components of the design, depending on what class of object `setStrata<-()` is applied to. When used on an object of class `"how"`, `setStrata<-()` replaces the `blocks` component of that object. When used on an object of class `"Plots"`, `setStrata<-()` replaces the `strata` component of that object. In both cases a factor variable is required and the replacement object will be coerced to a factor via `as.factor()` if possible.

`setType<-()`
 ~ replaces the `type` component of an object of class `"Plots"` or `"Within"` with a character vector of length one. Must be one of the available types: `"none"`, `"free"`, `"series"`, or `"grid"`.

`setMirror<-()`
 ~ replaces the `mirror` component of an object of class `"Plots"` or `"Within"` with a logical vector of length one.

`setConstant<-()`
 ~ replaces the `constant` component of an object of class `"Within"` with a logical vector of length one.

`setRow<-()`, `setCol<-()`, `setDim<-()`
 ~ replace one or both of the spatial grid dimensions of an object of class `"Plots"` or `"Within"` with am integer vector of length one, or, in the case of `setDim<-()`, of length 2.

`setNperm<-()`, `setMinperm<-()`, `setMaxperm<-()`
 ~ update the stored values for the requested number of permutations and the minimum and maximum permutation thresholds that control whether the entire set of permutations is generated instead of `nperm` permutations.

`setAllperms<-()`
 ~ assigns a matrix of permutation indices to the `all.perms` component of the design list object.

`setComplete<-()`
 ~ updates the status of the `complete` setting. Takes a logical vector of length 1 or any object coercible to such.

`setMake<-()`
 ~ sets the indicator controlling whether the entire set of permutations is generated during checking of the design via `check()`. Takes a logical vector of length 1 or any object coercible to such.

`setObserved<-()`
 ~ updates the indicator of whether the observed ordering is included in the set of all permutations should they be generated. Takes a logical vector of length 1 or any object coercible to such.

*Examples*

I illustrate the behaviour of the `getFoo()` and `setFoo<-()` functions through a couple of simple examples. Firstly, generate a design object
```{r, get-set-eg0}
hh <- how()
```
This design is one of complete randomization, so all of the settings in the object take their default values. The default number of permutations is currently `r getNperm(hh)`, and can be extracted using `getNperm()`
```{r, get-set-eg1}
getNperm(hh)
```
The corresponding replacement function can be use to alter the number of permutations after the design has been generated. To illustrate a finer point of the behaviour of these replacement functions, compare the matched call stored in `hh` before and after the number of permutations is changed
```{r, <get-set-eg2}
getCall(hh)
setNperm(hh) <- 999
getNperm(hh)
getCall(hh)
```
Note how the `call` component has been altered to include the argument pair `nperm = 999`, hence if this call were evaluated, the resulting object would be a copy of `hh`.

As a more complex example, consider the following design consisting of 5 blocks, each containing 2 plots of 5 samples each. Hence there are a total of 10 plots. Both the plots and within-plot sample are time series. This design can be created using
```{r, get-set-eg3}
hh <- how(within = Within(type = "series"),
          plots = Plots(type = "series", strata = gl(10, 5)),
          blocks = gl(5, 10))
```

To alter the design at the plot or within-plot levels, it is convenient to extract the relevant component using `getPlots()` or `getWithin()`, update the extracted object, and finally use the updated object to update `hh`. This process is illustrated below in order to change the plot-level permutation type to `"free"`
```{r, get-set-eg4}
pl <- getPlots(hh)
setType(pl) <- "free"
setPlots(hh) <- pl
```
We can confirm this has been changed by extracting the permutation type for the plot level
```{r, get-set-eg5}
getType(hh, which = "plots")
```
Notice too how the call has been expanded from `gl(10, 5)` to an integer vector. This expansion is to avoid the obvious problem of locating the objects referred to in the call should the call be re-evaluated later.
```{r, get-set-eg6}
getCall(getPlots(hh))
```

At the top level, a user can update the design using `update()`. Hence the equivalent of the above update is (this time resetting the original type; `type = "series"`)
```{r, get-set-eg7}
hh <- update(hh, plots = update(getPlots(hh), type = "series"))
getType(hh, which = "plots")
```
However, this approach is not assured of working within a function because we do not guarantee that components of the call used to create `hh` can be found from the execution frame where `update()` is called. To be safe, always use the `setFoo<-()` replacement functions to update design objects from within your functions.

# Computational details

This vignette was built within the following environment:
```{r, seesionInfo, echo=FALSE}
sessioninfo::session_info()
```

\name{shuffle-utils}
\alias{shuffle-utils}
\alias{shuffleFree}
\alias{shuffleGrid}
\alias{shuffleSeries}
\alias{shuffleStrata}

\title{Utility functions for unrestricted and restricted permutations}
\description{
  Unrestricted and restricted permutations for time series,
  line transects, spatial grids and blocking factors.
}
\usage{
shuffleFree(x, size)

shuffleSeries(x, mirror = FALSE, start = NULL, flip = NULL)

shuffleGrid(nrow, ncol, mirror = FALSE, start.row = NULL,
            start.col = NULL, flip = NULL)

shuffleStrata(strata, type, mirror = FALSE, start = NULL, flip = NULL,
              nrow, ncol, start.row = NULL, start.col = NULL)
}

\arguments{
  \item{x}{vector of indices to permute.}
  \item{size}{number of random permutations required}
  \item{mirror}{logical; should mirroring of sequences be allowed?}
  \item{start}{integer; the starting point for time series
    permutations. If missing, a random starting point is determined.}
  \item{flip}{logical, length 1 (\code{shuffleSeries}) or length 2
    (\code{shuffleGrid}); force mirroring of permutation. This will
    always return the reverse of the computed permutation. For
    \code{shuffleGrid}, the first element pertains to flipping rows, the
    second to flipping columns of the grid.}
  \item{nrow, ncol}{numeric; the number of rows and columns in the grid.}
  \item{start.row, start.col}{numeric; the starting row and column for
    the shifted grid permutation. If non supplied, a random starting row
    and column will be selected.}
  \item{strata}{factor; the blocks to permute.}
  \item{type}{character; the type of permutation used to shuffle the
    \code{strata}. One of \code{"free"}, \code{"grid"} or
    \code{"series"}.}
}
\details{
  These are developer-level functions for generating permuted indexes
  from one of several restricted and unrestricted designs.

  \code{shuffleFree} is a wrapper to code underlying
  \code{\link{sample}}, but without the extra over head of sanity
  checks. It is defined as \code{sample.int(x, size, replace = FALSE)}.
  You must arrange for the correct values to be supplied, where
  \code{x} is a vector of indices to sample from, and \code{size} is the
  number of indices to sample. Sampling is done without replacement and
  without regard to prior probabilities. Argument \code{size} is allowed
  so that one can draw a single observation at random from the indices
  \code{x}. In general use, \code{size} would be set equal to
  \code{length{x}}.
}
\value{
  A integer vector of permuted indices.
}
%\note{
%  \code{shuffle} is currently used in one Vegan function;
%  \code{\link[vegan]{permutest.betadisper}}. Over time, the other functions
%  that currently use the older \code{\link[vegan]{shuffle}} will be
%  updated to use \code{shuffle}.
%}
%\references{
%}
\author{Gavin Simpson}
\seealso{\code{\link{check}}, a utility function for checking
  permutation scheme described by
  \code{\link{how}}. \code{\link{shuffle}} as a user-oriented
  wrapper to these functions.}

\examples{
\dontshow{suppressWarnings(RNGversion("3.5.0"))}
set.seed(3)

## draw 1 value at random from the set 1:10
shuffleFree(1:10, 1)

## permute the series 1:10
x <- 1:10
shuffleSeries(x)                ## with random starting point
shuffleSeries(x, start = 5L)    ## known starting point
shuffleSeries(x, flip = TRUE)   ## random start, forced mirror
shuffleSeries(x, mirror = TRUE) ## random start, possibly mirror

## permute a grid of size 3x3
shuffleGrid(3, 3)                      ## random starting row/col
shuffleGrid(3, 3, start.row = 2,
            start.col = 3)             ## with known row/col
shuffleGrid(3, 3, flip = rep(TRUE, 2)) ## random start, forced mirror
}
\keyword{ htest }
\keyword{ design }

\name{check}
\alias{check}
\alias{print.check}
\alias{print.summary.check}
\alias{summary.check}

\title{Utility functions for permutation schemes}
\description{
  \code{check} provides checking of permutation schemes for
  validity. \code{permuplot} produces a graphical representation of the
  selected permutation design.
}
\usage{
check(object, control = how(), quietly = FALSE)

\method{summary}{check}(object, \dots)
}

\arguments{
  \item{object}{an R object. See Details for a complete description,
    especially for \code{\link{numPerms}}. For
    \code{\link{summary.check}} an object of class \code{"check"}.}
  \item{control}{a list of control values describing properties of the
    permutation design, as returned by a call to \code{\link{how}}.}
  \item{quietly}{logical; should messages by suppressed?}
  \item{\dots}{arguments to other methods.}
}
\details{
  \code{check} is a utility functions for working with the new
  permutation schemes available in \code{\link{shuffle}}.

  \code{check} is used to check the current permutation schemes
  against the object to which it will be applied. It calculates the
  maximum number of possible permutations for the number of observations
  in \code{object} and the permutation scheme described by
  \code{control}. The returned object contains component \code{control},
  an object of class \code{"how"} suitably modified if
  \code{check} identifies a problem.

  The main problem is requesting more permutations than is possible with
  the number of observations and the permutation design. In such cases,
  \code{nperm} is reduced to equal the number of possible permutations,
  and complete enumeration of all permutations is turned on
  (\code{control$complete} is set to \code{TRUE}). 

  Alternatively, if the number of possible permutations is low, and
  less than \code{control$minperm}, it is better to enumerate all
  possible permutations, and as such complete enumeration of all
  permutations is turned on (\code{control$complete} is set to
  \code{TRUE}). This guarantees that permutations are all unique and
  there are no duplicates.
}
\value{
  For \code{check} a list containing the maximum number of
  permutations possible and an object of class \code{"\link{how}"}.
}
\author{Gavin L. Simpson}
\seealso{\code{\link{shuffle}} and \code{\link{how}}.}

\examples{
## only run this example if vegan is available
if (suppressPackageStartupMessages(require("vegan"))) {
    ## use example data from ?pyrifos in package vegan
    example(pyrifos)

    ## Demonstrate the maximum number of permutations for the pyrifos data
    ## under a series of permutation schemes

    ## no restrictions - lots of perms
    CONTROL <- how(within = Within(type = "free"))
    (check1 <- check(pyrifos, CONTROL))
    ## summary(check1)
    
    ## no strata but data are series with no mirroring, so 132 permutations
    CONTROL <- how(within = Within(type = "series", mirror = FALSE))
    check(pyrifos, CONTROL)
    
    ## no strata but data are series with mirroring, so 264 permutations
    CONTROL <- how(within = Within(type = "series", mirror = TRUE))
    check(pyrifos, control = CONTROL)
    
    ## unrestricted within strata
    check(pyrifos, control = how(plots = Plots(strata = ditch),
                   within = Within(type = "free")))
    
    ## time series within strata, no mirroring
    check(pyrifos,
          control = how(plots = Plots(strata = ditch),
          within = Within(type = "series", mirror = FALSE)))
    
    ## time series within strata, with mirroring
    check(pyrifos,
          control = how(plots = Plots(strata = ditch),
          within = Within(type = "series", mirror = TRUE)))
    
    ## time series within strata, no mirroring, same permutation
    ## within strata
    check(pyrifos,
          control = how(plots = Plots(strata = ditch),
          within = Within(type = "series", constant = TRUE)))
    
    ## time series within strata, with mirroring, same permutation
    ## within strata
    check(pyrifos,
          control = how(plots = Plots(strata = ditch),
          within = Within(type = "series", mirror = TRUE,
          constant = TRUE)))
    ## permute strata
    check(pyrifos, how(plots = Plots(strata = ditch, type = "free"),
                       within = Within(type = "none")))
}
    
## this should also also for arbitrary vectors
vec1 <- check(1:100)
vec2 <- check(1:100, how())
all.equal(vec1, vec2)
vec3 <- check(1:100, how(within = Within(type = "series")))
all.equal(100, vec3$n)
vec4 <- check(1:100, how(within = Within(type= "series", mirror = TRUE)))
all.equal(vec4$n, 200)

## enumerate all possible permutations
fac <- gl(2,6)
ctrl <- how(plots = Plots(strata = fac),
            within = Within(type = "grid", mirror = FALSE,
                            constant = TRUE, nrow = 3, ncol = 2))
check(1:12, ctrl)

numPerms(1:12, control = ctrl)
(tmp <- allPerms(12, control = update(ctrl, observed = TRUE)))
(tmp2 <- allPerms(12, control = ctrl))

## turn on mirroring
ctrl <- update(ctrl, within = update(getWithin(ctrl), mirror = TRUE))
numPerms(1:12, control = ctrl)
(tmp3 <- allPerms(12, control = update(ctrl, observed = TRUE)))
(tmp4 <- allPerms(12, control = ctrl))
## prints out details of the permutation scheme as
## well as the matrix of permutations
summary(tmp)
summary(tmp2)

## different numbers of observations per level of strata
fac <- factor(rep(1:3, times = c(3,2,2)))
## free permutations in levels of strata
numPerms(7, how(within = Within(type = "free"),
                plots = Plots(strata = fac, type = "none")))
allPerms(7, how(within = Within(type = "free"),
                plots = Plots(strata = fac)))
## series permutations in levels of strata
ctrl <- how(within = Within(type = "series"), plots = Plots(strata = fac))
numPerms(7, control = ctrl)
allPerms(7, control = ctrl)
}
\keyword{ utilities }
\keyword{ design }
\keyword{ methods }

\name{nobs-methods}
\alias{nobs-methods}
\alias{nobs.numeric}
\alias{nobs.integer}
\alias{nobs.matrix}
\alias{nobs.data.frame}
\alias{nobs.factor}
\alias{nobs.character}

\title{Number of observations in a given object}
\description{
  \code{\link{nobs}} is a generic function to return the number of
  observations from a model. \code{shuffle} provides a few methods for
  other types of data object in R.
}
\usage{

\method{nobs}{numeric}(object, \dots)

\method{nobs}{integer}(object, \dots)

\method{nobs}{matrix}(object, \dots)

\method{nobs}{data.frame}(object, \dots)

\method{nobs}{character}(object, \dots)

\method{nobs}{factor}(object, \dots)
}
\arguments{
  \item{object}{a data frame or matrix, or a numeric, integer,
    character, or factor vector.}
  \item{\dots}{arguments to other methods.}
}
\details{
  Function \code{nobs} is a simple generic function to return the
  number of observations in a range of R model objects. Methods are
  provided to work with a variety of R objects.
}
\value{
  The (numeric) number of observations in \code{object}.
}
\author{Gavin Simpson}
\examples{
\dontshow{suppressWarnings(RNGversion("3.5.0"))}
set.seed(1)
## numeric vector
len <- sample(1:10, 1)
v <- as.numeric(sample(1:100, len))
len
obs <- nobs(v)
isTRUE(all.equal(len, obs))

## integer
len <- sample(1L:10L, 1)
obs <- nobs(len)
isTRUE(all.equal(len, obs))

}

\name{get-methods}
\alias{get-methods}
\alias{getBlocks}
\alias{getBlocks.default}
\alias{getBlocks.how}
\alias{getBlocks.permControl}
\alias{getWithin}
\alias{getWithin.default}
\alias{getWithin.how}
\alias{getWithin.permControl}
\alias{getStrata}
\alias{getStrata.default}
\alias{getStrata.how}
\alias{getStrata.permControl}
\alias{getStrata.Plots}
\alias{getType}
\alias{getType.default}
\alias{getType.how}
\alias{getType.permControl}
\alias{getType.Plots}
\alias{getType.Within}
\alias{getMirror}
\alias{getMirror.default}
\alias{getMirror.how}
\alias{getMirror.permControl}
\alias{getMirror.Plots}
\alias{getMirror.Within}
\alias{getConstant}
\alias{getConstant.default}
\alias{getConstant.how}
\alias{getConstant.permControl}
\alias{getConstant.Within}
\alias{getPlots}
\alias{getPlots.default}
\alias{getPlots.how}
\alias{getPlots.permControl}
\alias{getRow}
\alias{getRow.default}
\alias{getRow.how}
\alias{getRow.permControl}
\alias{getRow.Plots}
\alias{getRow.Within}
\alias{getCol}
\alias{getCol.default}
\alias{getCol.how}
\alias{getCol.permControl}
\alias{getCol.Plots}
\alias{getCol.Within}
\alias{getDim}
\alias{getDim.default}
\alias{getDim.how}
\alias{getDim.permControl}
\alias{getDim.Plots}
\alias{getDim.Within}
\alias{getNperm}
\alias{getNperm.default}
\alias{getNperm.how}
\alias{getNperm.permControl}
\alias{getMaxperm}
\alias{getMaxperm.default}
\alias{getMaxperm.how}
\alias{getMaxperm.permControl}
\alias{getMinperm}
\alias{getMinperm.default}
\alias{getMinperm.how}
\alias{getMinperm.permControl}
\alias{getComplete}
\alias{getComplete.default}
\alias{getComplete.how}
\alias{getComplete.permControl}
\alias{getMake}
\alias{getMake.default}
\alias{getMake.how}
\alias{getObserved}
\alias{getObserved.default}
\alias{getObserved.how}
\alias{getAllperms}
\alias{getAllperms.default}
\alias{getAllperms.how}
\alias{getControl}
\alias{getHow}
\alias{getControl.default}
\alias{getControl.allPerms}

\title{Extractor functions to access components of a permutation design}
\description{
  Simple functions to allow abstracted access to components of a
  permutation design, for example as returned by
  \code{\link{how}}. Whilst many of these are very simple index
  operations on a list, using these rather than directly accessing that
  list allows the internal representation of the permutation design to
  change without breaking code.
}
\usage{

getAllperms(object, ...)
getBlocks(object, ...)
getComplete(object, ...)
getConstant(object, ...)
getCol(object, ...)
getDim(object, ...)
getMake(object, ...)
getMaxperm(object, ...)
getMinperm(object, ...)
getMirror(object, ...)
getNperm(object, ...)
getObserved(object, ...)
getPlots(object, ...)
getRow(object, ...)
getStrata(object, ...)
getType(object, ...)
getWithin(object, ...)
getControl(object, ...)
getHow(object, ...)

\method{getAllperms}{how}(object, ...)

\method{getBlocks}{how}(object, ...)

\method{getCol}{how}(object, which = c("plots", "within"), ...)
\method{getCol}{Plots}(object, ...)
\method{getCol}{Within}(object, ...)

\method{getComplete}{how}(object, ...)

\method{getConstant}{how}(object, ...)
\method{getConstant}{Within}(object, ...)

\method{getDim}{how}(object, which = c("plots", "within"), ...)
\method{getDim}{Plots}(object, ...)
\method{getDim}{Within}(object, ...)

\method{getMake}{how}(object, ...)

\method{getMaxperm}{how}(object, ...)

\method{getMinperm}{how}(object, ...)

\method{getMirror}{how}(object, which = c("plots", "within"), ...)
\method{getMirror}{Plots}(object, ...)
\method{getMirror}{Within}(object, ...)

\method{getNperm}{how}(object, ...)

\method{getObserved}{how}(object, ...)

\method{getPlots}{how}(object, ...)

\method{getRow}{how}(object, which = c("plots", "within"), ...)
\method{getRow}{Plots}(object, ...)
\method{getRow}{Within}(object, ...)

\method{getStrata}{how}(object, which = c("plots", "blocks"),
          drop = TRUE, ...)
\method{getStrata}{Plots}(object, drop = TRUE, ...)

\method{getType}{how}(object, which = c("plots", "within"), ...)
\method{getType}{Plots}(object, ...)
\method{getType}{Within}(object, ...)

\method{getWithin}{how}(object, ...)

\method{getControl}{allPerms}(object, ...)
}

\arguments{
  \item{object}{An R object to dispatch on.}
  \item{which}{character; which level of restriction to extract
    information for.}
  \item{drop}{logical; should un-used factor levels be dropped?}
  \item{\dots}{Arguments passed on to other methods.}
}
\details{
  These are extractor functions for working with permutation design
  objects created by \code{\link{how}}. They should be used in
  preference to directly subsetting the permutation design in case the
  internal structure of object changes as \pkg{permute} is developed.

  \code{getHow} is an alias for \code{getControl}; specific methods are
  implemented for \code{getControl} if you are debugging.
}
\value{
  These are simple extractor functions and return the contents of the
  corresponding components of \code{object}.
}
\author{Gavin Simpson}
\seealso{\code{\link{check}}, a utility function for checking
  permutation scheme described by \code{\link{how}}.
}

\examples{
## extract components from a "how" object
hh <- how()
getWithin(hh)
getNperm(hh)
}
\keyword{ methods }
\keyword{ utils }

\name{set-methods}
\alias{set-methods}
\alias{setBlocks<-}
\alias{setBlocks<-.default}
\alias{setBlocks<-.how}
\alias{setBlocks<-.permControl}
\alias{setWithin<-}
\alias{setWithin<-.default}
\alias{setWithin<-.how}
\alias{setStrata<-}
\alias{setStrata<-.default}
\alias{setStrata<-.how}
\alias{setStrata<-.Plots}
\alias{setType<-}
\alias{setType<-.default}
\alias{setType<-.how}
\alias{setType<-.Plots}
\alias{setType<-.Within}
\alias{setMirror<-}
\alias{setMirror<-.default}
\alias{setMirror<-.how}
\alias{setMirror<-.Plots}
\alias{setMirror<-.Within}
\alias{setConstant<-}
\alias{setConstant<-.default}
\alias{setConstant<-.how}
\alias{setConstant<-.Plots}
\alias{setConstant<-.Within}
\alias{setPlots<-}
\alias{setPlots<-.default}
\alias{setPlots<-.how}
\alias{setRow<-}
\alias{setRow<-.default}
\alias{setRow<-.how}
\alias{setRow<-.Plots}
\alias{setRow<-.Within}
\alias{setCol<-}
\alias{setCol<-.default}
\alias{setCol<-.how}
\alias{setCol<-.Plots}
\alias{setCol<-.Within}
\alias{setDim<-}
\alias{setDim<-.default}
\alias{setDim<-.how}
\alias{setDim<-.Plots}
\alias{setDim<-.Within}
\alias{setNperm<-}
\alias{setNperm<-.default}
\alias{setNperm<-.how}
\alias{setNperm<-.permControl}
\alias{setAllperms<-}
\alias{setAllperms<-.default}
\alias{setAllperms<-.how}
\alias{setAllperms<-.permControl}
\alias{setMaxperm<-}
\alias{setMaxperm<-.default}
\alias{setMaxperm<-.how}
\alias{setMaxperm<-.permControl}
\alias{setMinperm<-}
\alias{setMinperm<-.default}
\alias{setMinperm<-.how}
\alias{setMinperm<-.permControl}
\alias{setComplete<-}
\alias{setComplete<-.default}
\alias{setComplete<-.how}
\alias{setComplete<-.permControl}
\alias{setMake<-}
\alias{setMake<-.default}
\alias{setMake<-.how}
\alias{setObserved<-}
\alias{setObserved<-.default}
\alias{setObserved<-.how}

\title{Replacement functions to set components of a permutation design}
\description{
  Simple functions to allow abstracted replacement of components of a
  permutation design, for example as returned by \code{\link{how}}. In
  addition to performing replacement of components of the list returned
  by \code{\link{how}}, these replacement function also update the
  matched calls stored within the list to facilitate the use of
  \code{\link{update}} by users.
}
\usage{

setBlocks(object) <- value
setPlots(object) <- value
setWithin(object) <- value
setStrata(object) <- value
setNperm(object) <- value
setAllperms(object) <- value
setMaxperm(object) <- value
setMinperm(object) <- value
setComplete(object) <- value
setMake(object) <- value
setObserved(object) <- value
setRow(object) <- value
setCol(object) <- value
setDim(object) <- value
setType(object) <- value
setMirror(object) <- value
setConstant(object) <- value
}

\arguments{
  \item{object}{An R object to dispatch on.}
  \item{value}{The replacement value/object.}
}
\details{
  These are replacement functions for working with permutation design
  objects created by \code{\link{how}}. They should be used in
  preference to directly updating the permutation design in case the
  internal structure of object changes as \pkg{permute} is developed and
  because the matched call also needs to be updated to facilitate use of
  \code{\link{update}} on the \code{\link{how}} object.
}
\section{Note}{
  \code{setStrata<-} has methods for objects of class \code{"how"} and
  \code{"Plots"}. The former sets the \code{blocks} component of the
  \code{\link{how}} object, whilst the latter sets the \code{strata}
  component of the \code{\link{Plots}} object.

  \code{setDim<-}, \code{setRow<-}, and \code{setCol<-} cannot be used
  on an object of class \code{"how"}. Instead, extract the \code{Plots}
  or \code{Within} components with \code{\link{getPlots}} or
  \code{\link{getWithin}} and alter those components, then use the
  resulting object to replace the \code{plots} or \code{within}
  components using \code{setPlots} or \code{setWithin}.
}
\value{
  These replacement functions return \code{object} suitably modified.
}
\author{Gavin Simpson}
\seealso{\code{\link{check}}, a utility function for checking
  permutation scheme described by \code{\link{how}}. Comparable
  extractor functions are also available; see
  \code{\link{get-methods}}.
}

\examples{
## extract components from a "how" object
hh <- how()
getNperm(hh)
setNperm(hh) <- 999
getNperm(hh)
}
\keyword{ methods }
\keyword{ utils }

\name{allPerms}
\alias{allPerms}
\alias{print.allPerms}
\alias{summary.allPerms}
\alias{print.summary.allPerms}
\alias{as.matrix.allPerms}
\alias{as.allPerms}

\title{Complete enumeration of all possible permutations}
\description{
  \code{allPerms} is a utility function to return the set of
  permutations for a given R object and a specified permutation design.
}
\usage{
allPerms(n, control = how(), check = TRUE)

\method{summary}{allPerms}(object, \dots)

\method{as.matrix}{allPerms}(x, \dots)

as.allPerms(object, control)
}
\arguments{
  \item{n}{the number of observations or an 'object' from which the
    number of observations can be determined via \code{getNumObs}.}
  \item{control}{a list of control values describing properties of the
    permutation design, as returned by a call to
    \code{\link{how}}.}
  \item{check}{logical; should \code{allPerms} check the design? The
    default is to check, but this can be skipped, for example if a
    function checked the design earlier.}
  \item{object}{for \code{summary.allPerms}, an object of class
    \code{"allPerms"}. For \code{as.allPerms} a matrix or something that
    can be coerced to a matrix by \code{\link{as.matrix}}.}
  \item{\dots}{arguments to other methods.}
  \item{x}{
    an object of class \code{"allPerms"}, as returned by \code{allPerms}.
  }
}
\details{
  Function \code{allPerms} enumerates all possible permutations for the 
  number of observations and the selected permutation scheme. It has
  \code{\link{print}} and \code{\link{summary}} methods. \code{allPerms}
  returns a matrix containing all possible permutations, possibly
  containing the observed ordering (if argument \code{observed} is
  \code{TRUE}). The rows of this matrix are the various permutations and
  the columns reflect the number of samples.

  With free permutation designs, and restricted permutation schemes with
  large numbers of observations, there are a potentially huge number of
  possible permutations of the samples. It would be inefficient, not to
  mention incredibly time consuming, to enumerate them all. Storing all
  possible permutations would also become problematic in such cases. To
  control this and guard against trying to evaluate too large a number
  of permutations, if the number of possible permutations is larger than
  \code{getMaxperm(control)}, \code{allPerms} exits with an error.

  The \code{as.matrix} method sets the \code{control} and \code{seed}
  attributes to \code{NULL} and removes the \code{"permutationMatrix"}
  class, resulting in a standard matrix object.
}
\value{
  For \code{allPerms}, and object of class \code{"allPerms"}, a matrix
  whose rows are the set of all possible permutations for the supplies
  number of observations and permutation scheme selected. The matrix has
  two additional attributes \code{control} and
  \code{observed}. Attribute \code{control} contains the argument
  \code{control} (possibly updated via \code{check}). Attribute 
  \code{observed} contains argument \code{observed}.
}
\section{Warning}{
  If permuting the strata themselves, a balanced design is required (the
  same number of observations in each level of \code{strata}. This is
  common to all functions in the package.
}
\author{Gavin Simpson}
\examples{
## allPerms can work with a vector
vec <- c(3,4,5)
allPerms(vec) ## free permutation

## enumerate all possible permutations for a more complicated
## design
fac <- gl(2,6)
ctrl <- how(within = Within(type = "grid", mirror = FALSE,
                            constant = TRUE, nrow = 3, ncol = 2),
            plots = Plots(strata = fac))
Nobs <- length(fac)
numPerms(seq_len(Nobs), control = ctrl) ## 6
(tmp <- allPerms(Nobs, control = update(ctrl, observed = TRUE)))
(tmp2 <- allPerms(Nobs, control = ctrl))

## turn on mirroring
##ctrl$within$mirror <- TRUE
ctrl <- update(ctrl, within = update(getWithin(ctrl), mirror = TRUE))
numPerms(seq_len(Nobs), control = ctrl)
(tmp3 <- allPerms(Nobs, control = update(ctrl, observed = TRUE)))
(tmp4 <- allPerms(Nobs, control = ctrl))

## prints out details of the permutation scheme as
## well as the matrix of permutations
summary(tmp3)
summary(tmp4)
}

\name{allUtils}
\alias{allFree}
\alias{allSeries}
\alias{allGrid}
\alias{allStrata}

\title{Utility functions for complete enumeration of all possible
  permutations}

\description{
  Utility functions to return the set of all permutations under
  different designs. For most practical applications, i.e. to combine
  designs permuting blocks and/or within blocks function
  \code{\link{allPerms}} will be required.
}

\usage{
allFree(n, v = seq_len(n))

allSeries(n, nperms, mirror = FALSE)

allGrid(n, nperms, nr, nc, mirror, constant)

allStrata(n, control)
}

\arguments{
  \item{n}{the number of observations.}
  \item{v}{numeric; vector of indices. Default is \code{1:n}.}
  \item{nperms}{numeric; number of possible permutations.}
  \item{mirror}{logical; mirroring of permutations allowed?}
  \item{nr,nc}{integer; number of rows and columns of grid designs.}
  \item{constant}{logical; same permutation within each block?}
  \item{control}{a list of control values describing properties of the
    permutation design, as returned by a call to \code{\link{how}}.}
}

\details{
  These are utility functions and aren't designed for casual
  use. \code{\link{allPerms}} should be used instead.

  Details on usage of these functions can be found in
  \code{\link{allPerms}}.
}

\value{
  A matrix of all possible permutations of \code{n} observations or of
  \code{v}, given the provided options.
}

\author{Gavin Simpson}

\name{how}
\alias{how}
\alias{print.how}
\alias{Blocks}
\alias{Within}
\alias{Plots}

\title{How to define a permutation design?}

\description{
  Utility functions to describe unrestricted and restricted permutation
  designs for time series, line transects, spatial grids and blocking
  factors.
}

\usage{
how(within = Within(), plots = Plots(), blocks = NULL,
    nperm = 199, complete = FALSE, maxperm = 9999,
    minperm = 5040, all.perms = NULL, make = TRUE,
    observed = FALSE)

Within(type = c("free","series","grid","none"),
       constant = FALSE, mirror = FALSE,
       ncol = NULL, nrow = NULL)

Plots(strata = NULL, type = c("none","free","series","grid"),
      mirror = FALSE, ncol = NULL, nrow = NULL)
}

\arguments{
  \item{within, plots, blocks}{Permutation designs for samples within the
    levels of \code{plots} (\code{within}), permutation of \code{plots}
    themselves, or for the definition of blocking structures which
    further restrict permutations (\code{blocks}). \code{within} and
    \code{plots} each require a named list as produced by \code{Within}
    and \code{Plots} respectively. \code{blocks} takes a factor (or an
    object coercible to a factor via \code{as.factor}), the levels of
    which define the blocking structure.}
  \item{nperm}{numeric; the number of permutations.}
  \item{complete}{logical; should complete enumeration of all
    permutations be performed?}
  \item{type}{character; the type of permutations required. One of
    \code{"free"}, \code{"series"}, \code{"grid"} or \code{"none"}. See
    Details.} 
  \item{maxperm}{numeric; the maximum number of permutations to
    perform. Currently unused.}
  \item{minperm}{numeric; the lower limit to the number of possible
    permutations at which complete enumeration is performed. When
    \code{nperm} is lower than \code{minperm}, sampling is performed
    from the set of complete permutations to avoid duplicate
    permutations. See argument \code{complete} and Details, below.}
  \item{all.perms}{an object of class \code{allPerms}, the result of a
    call to \code{\link{allPerms}}.}
  \item{make}{logical; should \code{check} generate all possible
    permutations? Useful if want to check permutation design but not
    produce the matrix of all permutations, or to circumvent the
    heuristics governing when complete enumeration is activated.}
  \item{observed}{logical; should the observed permutation be returned
    as part of the set of all permutations? Default is \code{FALSE} to
    facilitate usage in higher level functions.}
  \item{constant}{logical; should the same permutation be used within
    each level of strata? If \code{FALSE} a separate, possibly restricted,
    permutation is produced for each level of \code{strata}.}
  \item{mirror}{logical; should mirroring of sequences be allowed?}
  \item{ncol, nrow}{numeric; the number of columns and rows of samples
    in the spatial grid respectively.}
  \item{strata}{A factor, or an object that can be coerced to a factor
    via \code{as.factor}, specifying the strata for permutation.} 
}
\details{
  \code{shuffle} can generate permutations for a wide range of
  restricted permutation schemes. A small selection of the available
  combinations of options is provided in the Examples section below.

  Argument type controls how samples are actually permuted;
  \code{"free"} indicates randomization, \code{"series"} indicates
  permutation via cyclic shifts (suitable for evenly-spaced line
  transect or time series data), \code{"grid"} indicates permutation via
  toroidal shifts (suitable for samples on a regular grid), and
  \code{"none"} indicates no permutation of samples. See the package
  vignette (\code{browseVignettes("permute")}) for additional
  information on each of these types of permutation.

  Argument \code{mirror} determines whether grid or series permutations
  can be mirrored. Consider the sequence 1,2,3,4. The relationship
  between consecutive observations is preserved if we reverse the
  sequence to 4,3,2,1. If there is no inherent direction in your
  experimental design, mirrored permutations can be considered
  part of the Null model, and as such increase the number of possible
  permutations. The default is to not use mirroring so you must
  explicitly turn this on using \code{mirror = TRUE} in \code{how}.

  To permute plots rather than the observations within plots (the
  levels of \code{strata}), use \code{Within(type = "none")} and
  \code{Plots(type = foo)}, where \code{foo} is how you want the plots
  to be permuted. However, note that the number of observations within
  each plot \strong{must} be equal! 

  For some experiments, such as BACI designs, one might wish to use the
  same permutation within each plot. This is controlled by
  argument \code{constant}. If \code{constant = TRUE} then the same
  permutation will be generated for each level of \code{strata}. The
  default is \code{constant = FALSE}.
}
\value{  
  For \code{how} a list with components for each of the possible arguments.
}
\references{
  \code{shuffle()} is modelled after the permutation schemes of Canoco
  3.1 (ter Braak, 1990); see also Besag & Clifford (1989).

  Besag, J. and Clifford, P. (1989) Generalized Monte Carlo significance
  tests. \emph{Biometrika} \strong{76}; 633--642.

  ter Braak, C. J. F. (1990). \emph{Update notes: CANOCO version
    3.1}. Wageningen: Agricultural Mathematics Group. (UR).
}
\author{Gavin Simpson}
\seealso{\code{\link{shuffle}} and \code{\link{shuffleSet}} for
  permuting from a design, and \code{\link{check}}, a utility function
  for checking permutation design described by \code{how}.}
\examples{
## Set up factors for the Plots and Blocks
plts <- gl(4, 10) ## 4 Plots of 10 samples each
blks <- gl(2, 20) ## 2 Blocks of 20 samples each

## permutation design
h1 <- how(within = Within(type = "series", mirror = TRUE),
          plots = Plots(strata = plts, type = "series"),
          blocks = blks)

## The design can be updated...
## ... remove the blocking:
update(h1, blocks = NULL)

## ... or switch the type of shuffling at a level:
#update(h1, plots = update(getPlots(h1), type = "none"))
plots2 <- update(getPlots(h1), type = "none")
update(h1, plots = plots2)
}
\keyword{ utils  }

\name{jackal}
\alias{jackal}
\docType{data}
\title{
  Mandible lengths of male and female golden jackals
}
\description{
  Mandible lengths (in mm) for male and female golden jackals
  (\emph{Canis aureus}) from a collection of specimens in the British
  Museum of Natural History, London, UK.
}
\usage{data(jackal)}
\format{
  A data frame with 20 observations on the following 2 variables.
  \describe{
    \item{\code{Length}}{a numeric vector}
    \item{\code{Sex}}{a factor with levels \code{Male} \code{Female}}
  }
}
%\details{
%
%}
\source{
  The data were manually transcribed from Manly (2007).
}
\references{
  Higham, C.F.W., Kijngam, A., and Manly, B.F.J. (1980) An analysis of
  prehistoric canid remains from Thailand. \emph{Journal of
    Archaeological Science} \strong{7}:149-165.
  
  Manly, B.F.J. (2007) \emph{Randomization, bootstrap and Monte Carlo
    methods in biology. Third Edition}. Chapman & Hall/CRC, Boca Raton.
}
\examples{
data(jackal)
str(jackal)

## boxplot of mandible length vs sex
plot(Length ~ Sex, data = jackal)
}
\keyword{datasets}

\name{numPerms}
\alias{numPerms}

\title{Number of possible permutations for a given object}
\description{
  \code{numPerms} calculates the maximum number of permutations possible
  under the current permutation scheme.
}
\usage{
numPerms(object, control = how())
}
\arguments{
  \item{object}{any object handled by \code{\link{nobs}}.}
  \item{control}{a list of control values describing properties of the
    permutation design, as returned by a call to
    \code{\link{how}}.}
}
\details{
  Function \code{numPerms} returns the number of permutations for the
  passed \code{object} and the selected permutation
  scheme. \code{object} can be one of a data frame, matrix, an object
  for which a scores method exists, or a numeric or integer vector. In
  the case of a numeric or integer vector, a vector of length 1 can be
  used and it will be expanded to a vector of length \code{object}
  (i.e., \code{1:object}) before computing the number of
  permutations. As such, \code{object} can be the number of observations
  not just the object containing the observations. 
}
\value{
  The (numeric) number of possible permutations of observations in
  \code{object}.
}
\note{
  In general, mirroring \code{"series"} or \code{"grid"} designs doubles
  or quadruples, respectively, the number of permutations without
  mirroring (within levels of strata if present). This is \strong{not}
  true in two special cases:
  \enumerate{
    \item In \code{"grid"} designs where the number of columns is equal
    to 2, and
    \item In \code{"series"} designs where the number of observations in
    a series is equal to 2.
  }

  For example, with 2 observations there are 2 permutations for
  \code{"series"} designs:
  \enumerate{
    \item 1-2, and
    \item 2-1.
  }
  If these two permutations were mirrored, we would have:
  \enumerate{
    \item 2-1, and
    \item 1-2.
  }
  It is immediately clear that this is the same set of permutations
  without mirroring (if one reorders the rows). A similar situation
  arises in \code{"grid"} designs where the number of \strong{columns}
  per \emph{grid} is equal to 2. Note that the number of rows per
  \emph{grid} is not an issue here.
}
\author{Gavin Simpson}
\seealso{\code{\link{shuffle}} and
  \code{\link{how}}. Additional \code{\link{nobs}} methods are
  provide, see \code{\link{nobs-methods}}.}
\examples{
## permutation design --- see ?how
ctrl <- how() ## defaults to freely exchangeable

## vector input
v <- 1:10
(obs <- nobs(v))
numPerms(v, control = ctrl)

## integer input
len <- length(v)
(obs <- nobs(len))
numPerms(len, control = ctrl)

## new design, objects are a time series
ctrl <- how(within = Within(type = "series"))
numPerms(v, control = ctrl)
## number of permutations possible drastically reduced...
## ...turn on mirroring
ctrl <- how(within = Within(type = "series", mirror = TRUE))
numPerms(v, control = ctrl)

## Try blocking --- 2 groups of 5
bl <- numPerms(v, control = how(blocks = gl(2,5)))
bl

## should be same as
pl <- numPerms(v, control = how(plots = Plots(strata = gl(2,5))))
pl
stopifnot(all.equal(bl, pl))
}

\name{shuffle}
\alias{shuffle}
\alias{permute}

\title{Unrestricted and restricted permutations}
\description{
  Unrestricted and restricted permutation designs for time series,
  line transects, spatial grids and blocking factors.
}
\usage{
shuffle(n, control = how())

permute(i, n, control)
}

\arguments{
  \item{n}{numeric; the length of the returned vector of permuted
    values. Usually the number of observations under consideration. May
    also be any object that \code{nobs} knows about; see
    \code{\link{nobs-methods}}.}
  \item{control}{a list of control values describing properties of the
    permutation design, as returned by a call to \code{how}.}
  \item{i}{integer; row of \code{control$all.perms} to return.}
}
\details{
  \code{shuffle} can generate permutations for a wide range of
  restricted permutation schemes. A small selection of the available
  combinations of options is provided in the Examples section below.

  \code{permute} is a higher level utility function for use in a loop
  within a function implementing a permutation test. The main purpose of
  \code{permute} is to return the correct permutation in each iteration
  of the loop, either a random permutation from the current design or
  the next permutation from \code{control$all.perms} if it is not
  \code{NULL} and \code{control$complete} is \code{TRUE}.
}
\value{
  For \code{shuffle} a vector of length \code{n} containing a
  permutation of the observations 1, \ldots, n using the permutation
  scheme described by argument \code{control}.
  
  For \code{permute} the \code{i}th permutation from the set of all
  permutations, or a random permutation from the design.
}
%\note{
%  \code{shuffle} is currently used in one Vegan function;
%  \code{\link[vegan]{permutest.betadisper}}. Over time, the other functions
%  that currently use the older \code{\link[vegan]{shuffle}} will be
%  updated to use \code{shuffle}.
%}
\references{
  \code{shuffle()} is modelled after the permutation schemes of Canoco
  3.1 (ter Braak, 1990); see also Besag & Clifford (1989).

  Besag, J. and Clifford, P. (1989) Generalized Monte Carlo significance
  tests. \emph{Biometrika} \strong{76}; 633--642.

  ter Braak, C. J. F. (1990). \emph{Update notes: CANOCO version
    3.1}. Wageningen: Agricultural Mathematics Group. (UR).
}
\author{Gavin Simpson}
\seealso{\code{\link{check}}, a utility function for checking
  permutation scheme described by \code{\link{how}}.}

\examples{
\dontshow{suppressWarnings(RNGversion("3.5.0"))}
set.seed(1234)

## unrestricted permutations
shuffle(20)

## observations represent a time series of line transect
CTRL <- how(within = Within(type = "series"))
shuffle(20, control = CTRL)

## observations represent a time series of line transect
## but with mirroring allowed
CTRL <- how(within = Within(type = "series", mirror = TRUE))
shuffle(20, control = CTRL)

## observations represent a spatial grid, 5rx4c
nr <- 5
nc <- 4
CTRL <- how(within = Within(type = "grid", ncol = nc, nrow = nr))
perms <- shuffle(20, control = CTRL)
## view the permutation as a grid
matrix(matrix(1:20, nrow = nr, ncol = nc)[perms],
       ncol = nc, nrow = nr)

## random permutations in presence of strata
plots <- Plots(strata = gl(4, 5))
CTRL <- how(plots = plots, within = Within(type = "free"))
shuffle(20, CTRL)
## as above but same random permutation within strata
CTRL <- how(plots = plots, within = Within(type = "free",
            constant = TRUE))
shuffle(20, CTRL)

## time series within each level of block
CTRL <- how(plots = plots, within = Within(type = "series"))
shuffle(20, CTRL)
## as above, but  with same permutation for each level
CTRL <- how(plots = plots, within = Within(type = "series",
            constant = TRUE))
shuffle(20, CTRL)

## spatial grids within each level of block, 4 x (5r x 5c)
nr <- 5
nc <- 5
nb <- 4 ## number of blocks
plots <- Plots(gl(nb, 25))
CTRL <- how(plots = plots,
            within = Within(type = "grid", ncol = nc, nrow = nr))
shuffle(100, CTRL)
## as above, but with same permutation for each level
CTRL <- how(plots = plots,
            within = Within(type = "grid", ncol = nc, nrow = nr,
                            constant = TRUE))
shuffle(100, CTRL)

## permuting levels of plots instead of observations
CTRL <- how(plots = Plots(gl(4, 5), type = "free"),
            within = Within(type = "none"))
shuffle(20, CTRL)
## permuting levels of plots instead of observations
## but plots represent a time series
CTRL <- how(plots = Plots(gl(4, 5), type = "series"),
            within = Within(type = "none"))
shuffle(20, CTRL)

## permuting levels of plots but plots represent a time series
## free permutation within plots
CTRL <- how(plots = Plots(gl(4, 5), type = "series"),
            within = Within(type = "free"))
shuffle(20, CTRL)

## permuting within blocks
grp <- gl(2, 10) # 2 groups of 10 samples each
CTRL <- how(blocks = grp)
shuffle(length(grp), control = CTRL)

## Simple function using permute() to assess significance
## of a t.test  
pt.test <- function(x, group, control) {
    ## function to calculate t
    t.statistic <- function(x, y) {
        m <- length(x)
        n <- length(y)
        ## means and variances, but for speed
        xbar <- mean(x)
        ybar <- mean(y)
        xvar <- var(x)
        yvar <- var(y)
        pooled <- sqrt(((m-1)*xvar + (n-1)*yvar) / (m+n-2))
        (xbar - ybar) / (pooled * sqrt(1/m + 1/n))
    }
    ## check the control object
    #control <- check(x, control)$control ## FIXME
    ## number of observations
    Nobs <- nobs(x)
    ## group names
    lev <- names(table(group))
    ## vector to hold results, +1 because of observed t
    t.permu <- numeric(length = control$nperm) + 1
    ## calculate observed t
    t.permu[1] <- t.statistic(x[group == lev[1]], x[group == lev[2]])
    ## generate randomisation distribution of t
    for(i in seq_along(t.permu)) {
        ## return a permutation
        want <- permute(i, Nobs, control)
        ## calculate permuted t
        t.permu[i+1] <- t.statistic(x[want][group == lev[1]],
                                    x[want][group == lev[2]])
    }
    ## pval from permutation test
    pval <- sum(abs(t.permu) >= abs(t.permu[1])) / (control$nperm + 1)
    ## return value
    return(list(t.stat = t.permu[1], pval = pval))
}

## generate some data with slightly different means
set.seed(1234)
gr1 <- rnorm(20, mean = 9)
gr2 <- rnorm(20, mean = 10)
dat <- c(gr1, gr2)
## grouping variable
grp <- gl(2, 20, labels = paste("Group", 1:2))
## create the permutation design
control <- how(nperm = 999, within = Within(type = "free"))
## perform permutation t test
perm.val <- pt.test(dat, grp, control)
perm.val

## compare perm.val with the p-value from t.test()
t.test(dat ~ grp, var.equal = TRUE)
}
\keyword{ htest }
\keyword{ design }

\name{shuffleSet}
\alias{shuffleSet}
\alias{as.matrix.permutationMatrix}

\title{
  Generate a set of permutations from the specified design.
}
\description{
  \code{shuffleSet} returns a set of \code{nset} permutations from the
  specified design. The main purpose of the function is to circumvent
  the overhead of repeatedly calling \code{\link{shuffle}} to generate a
  set of permutations.
}
\usage{
shuffleSet(n, nset, control = how(), check = TRUE, quietly = FALSE)

\method{as.matrix}{permutationMatrix}(x, \dots)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{n}{
    numeric; the number of observations in the sample set. May also be
    any object that \code{nobs} knows about; see
    \code{\link{nobs-methods}}.
  }
  \item{nset}{
    numeric; the number of permutations to generate for the set. Can be
    missing, the default, in which case \code{nset} is determined from
    \code{control}.
  }
  \item{control}{
    an object of class \code{"how"} describing a valid permutation
    design.
  }
  \item{check}{
    logical; should the design be checked for various problems via
    function \code{\link{check}}? The default is to check the design for
    the stated number of observations and update \code{control}
    accordingly. See Details.
  }
  \item{quietly}{
    logical; should messages by suppressed?
  }
  \item{x}{
    an object of class \code{"permutationMatrix"}, as returned by
    \code{shuffleSet}.
  }
  \item{\dots}{
    arguments passed to other methods. For the \code{as.matrix} method
    only.
  }
}
\details{
  \code{shuffleSet} is designed to generate a set of \code{nset}
  permutation indices over which a function can iterate as part of a
  permutation test. It is only slightly more efficient than calling
  \code{\link{shuffle}} \code{nset} times, but it is far more practical
  than the simpler function because a set of permutations can be worked
  on by applying a function to the rows of the returned object. This
  simplifies the function applied, and facilitates the use of parallel
  processing functions, thus enabling a larger number of permutations to
  be evaluated in reasonable time.

  By default, \code{shuffleSet} will check the permutations design
  following a few simple heuristics. See \code{\link{check}} for details
  of these. Whether some of the heuristics are activated or not can be
  controlled via \code{\link{how}}, essentially via its argument
  \code{minperm}. In particular, if there are fewer than \code{minperm}
  permutations, \code{shuffleSet} will generate and return \strong{all
    possible permutations}, which may differ from the number requested via
  argument \code{nset}.

  The \code{check} argument to \code{shuffleSet} controls whether
  checking is performed in the permutation design. If you set
  \code{check = FALSE} then exactly \code{nset} permutations will be
  returned. However, do be aware that there is no guarantee that the set
  of permutations returned will be unique, especially so for designs and
  data sets where there are few possible permutations relative to the
  number requested.

  The \code{as.matrix} method sets the \code{control} and \code{seed}
  attributes to \code{NULL} and removes the \code{"permutationMatrix"}
  class, resulting in a standard matrix object.
}
\value{
  Returns a matrix of permutations, where each row is a separate
  permutation. As such, the returned matrix has \code{nset} rows and
  \code{n} columns.
}
\author{
  Gavin L. Simpson
}

%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~
\references{
  \code{shuffleSet()} is modelled after the permutation schemes of Canoco
  3.1 (ter Braak, 1990); see also Besag & Clifford (1989).

  Besag, J. and Clifford, P. (1989) Generalized Monte Carlo significance
  tests. \emph{Biometrika} \strong{76}; 633--642.

  ter Braak, C. J. F. (1990). \emph{Update notes: CANOCO version
    3.1}. Wageningen: Agricultural Mathematics Group. (UR).
}
\seealso{
  See \code{\link{shuffle}} for generating a single permutation, and
  \code{\link{how}} for setting up permutation designs.
}
\examples{
\dontshow{suppressWarnings(RNGversion("3.5.0"))}
set.seed(1)
## simple random permutations, 5 permutations in set
shuffleSet(n = 10, nset = 5)

## series random permutations, 5 permutations in set
shuffleSet(10, 5, how(within = Within(type = "series")))

## series random permutations, 10 permutations in set,
## with possible mirroring
CTRL <- how(within = Within(type = "series", mirror = TRUE))
shuffleSet(10, 10, CTRL)

## Permuting strata
## 4 groups of 5 observations
CTRL <- how(within = Within(type = "none"),
            plots = Plots(strata = gl(4,5), type = "free"))
shuffleSet(20, 10, control = CTRL)

## 10 random permutations in presence of Plot-level strata
plotStrata <- Plots(strata = gl(4,5))
CTRL <- how(plots = plotStrata,
            within = Within(type = "free"))
numPerms(20, control = CTRL)
shuffleSet(20, 10, control = CTRL)
## as above but same random permutation within Plot-level strata
CTRL <- how(plots = plotStrata,
            within = Within(type = "free", constant = TRUE))
numPerms(20, control = CTRL)
shuffleSet(20, 10, CTRL) ## check this.

## time series within each level of Plot strata
CTRL <- how(plots = plotStrata,
            within = Within(type = "series"))
shuffleSet(20, 10, CTRL)
## as above, but  with same permutation for each Plot-level stratum
CTRL <- how(plots = plotStrata,
            within = Within(type = "series", constant = TRUE))
shuffleSet(20, 10, CTRL)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ htest }
\keyword{ design }
